
import re

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from customeditor import CustomEditor, editingFinished
from camelot.view.art import Icon
import camelot.types

class VirtualAddressEditor(CustomEditor):
  
    def __init__(self, parent=None, editable=True, **kwargs):
        CustomEditor.__init__(self, parent)
        self.layout = QtGui.QHBoxLayout()
        self.layout.setMargin(0)
        self.combo = QtGui.QComboBox()
        self.combo.addItems(camelot.types.VirtualAddress.virtual_address_types)
        self.combo.setEnabled(editable)
        self.layout.addWidget(self.combo)
        self.editor = QtGui.QLineEdit()
        self.editor.setEnabled(editable)
        self.layout.addWidget(self.editor)
        self.setFocusProxy(self.editor)
        self.editable = editable
        nullIcon = Icon('tango/16x16/devices/phone.png').getQIcon()
        self.label = QtGui.QToolButton()
        self.label.setIcon(nullIcon)
        self.label.setAutoFillBackground(False)
        self.label.setAutoRaise(True)
        self.label.setEnabled(False)
        self.label.setToolButtonStyle(Qt.ToolButtonIconOnly)
        
        self.layout.addWidget(self.label)    
        self.connect(self.editor,
                     QtCore.SIGNAL('editingFinished()'),
                     self.editingFinished)
        self.connect(self.editor,
                     QtCore.SIGNAL('textEdited(const QString&)'),
                     self.editorValueChanged)
        self.connect(self.combo,
                     QtCore.SIGNAL('currentIndexChanged(int)'),
                     lambda:self.comboIndexChanged())
        self.setLayout(self.layout)
        self.setAutoFillBackground(True)
        self.checkValue(self.editor.text())
    
    def comboIndexChanged(self):
        self.checkValue(self.editor.text())
        self.editingFinished()
        
    def set_value(self, value):
        value = CustomEditor.set_value(self, value)
        if value:
            self.editor.setText(value[1])
            idx = camelot.types.VirtualAddress.virtual_address_types.index(value[0])
            self.combo.setCurrentIndex(idx)
            if str(self.combo.currentText()) == 'phone':
                icon = Icon('tango/16x16/devices/phone.png').getQIcon()
            if str(self.combo.currentText()) == 'fax':
                icon = Icon('tango/16x16/devices/printer.png').getQIcon()
            if str(self.combo.currentText()) == 'mobile':
                icon = Icon('tango/16x16/devices/mobile.png').getQIcon()
            if str(self.combo.currentText()) == 'im':
                icon = Icon('tango/16x16/places/instant-messaging.png').getQIcon()
            if str(self.combo.currentText()) == 'pager':
                icon = Icon('tango/16x16/devices/pager.png').getQIcon()
            if str(self.combo.currentText()) == 'email':
                icon = Icon('tango/16x16/apps/internet-mail.png').getQIcon()
                self.label.setFocusPolicy(Qt.StrongFocus)
                self.label.setAutoRaise(True)
                self.label.setAutoFillBackground(True)
                self.label.setIcon(icon)
                self.label.setEnabled(self.editable)
                self.label.setDisabled(not self.editable)
                self.connect(self.label,
                             QtCore.SIGNAL('clicked()'),
                             lambda:self.mailClick(self.editor.text()))
            else:
                self.label.setIcon(icon)
                self.label.setAutoFillBackground(False)
                self.label.setAutoRaise(True)
                self.label.setEnabled(False)
                self.label.setToolButtonStyle(Qt.ToolButtonIconOnly)
              
#      self.update()
#      self.label.update()
#      self.layout.update()
      
      
            self.checkValue(value[1])
      
    def get_value(self):
        value = (unicode(self.combo.currentText()), unicode(self.editor.text()))
        return CustomEditor.get_value(self) or value
      
    def set_enabled(self, editable=True):
        self.combo.setEnabled(editable)
        self.editor.setEnabled(editable)
        if not editable:
            self.label.setEnabled(False)
        else:
            if self.combo.currentText() == 'email':
                self.label.setEnabled(True)
          
        
    def checkValue(self, text):
        if self.combo.currentText() == 'email':
            email = unicode(text)
            mailCheck = re.compile('^\S+@\S+\.\S+$')
            if not mailCheck.match(email):
                palette = self.editor.palette()
                palette.setColor(QtGui.QPalette.Active,
                                 QtGui.QPalette.Base,
                                 QtGui.QColor(255, 0, 0))
                self.editor.setPalette(palette)
            else:
                palette = self.editor.palette()
                palette.setColor(QtGui.QPalette.Active,
                                 QtGui.QPalette.Base,
                                 QtGui.QColor(255, 255, 255))
                self.editor.setPalette(palette)
                
        elif self.combo.currentText() == 'phone' \
         or self.combo.currentText() == 'pager' \
         or self.combo.currentText() == 'fax' \
         or self.combo.currentText() == 'mobile':
    
            number = unicode(text)
            numberCheck = re.compile('^[0-9 ]+$')
      
            if not numberCheck.match(number):
                palette = self.editor.palette()
                palette.setColor(QtGui.QPalette.Active,
                                 QtGui.QPalette.Base,
                                 QtGui.QColor(255, 0, 0))
                self.editor.setPalette(palette)
            else:
                palette = self.editor.palette()
                palette.setColor(QtGui.QPalette.Active,
                                 QtGui.QPalette.Base,
                                 QtGui.QColor(255, 255, 255))
                self.editor.setPalette(palette)
                
        else:
            Check = re.compile('^.+$')
            if not Check.match(unicode(text)):
                palette = self.editor.palette()
                palette.setColor(QtGui.QPalette.Active,
                                 QtGui.QPalette.Base,
                                  QtGui.QColor(255, 0, 0))
                self.editor.setPalette(palette)
            else:
                palette = self.editor.palette()
                palette.setColor(QtGui.QPalette.Active,
                                  QtGui.QPalette.Base,
                                  QtGui.QColor(255, 255, 255))
                self.editor.setPalette(palette)
        
    def editorValueChanged(self, text):
        self.checkValue(text)
    
    def mailClick(self, adress):
        url = QtCore.QUrl()
        url.setUrl('mailto:%s?subject=Subject'%str(adress))
        QtGui.QDesktopServices.openUrl(url)
    
    def editingFinished(self):
        self.value = []
        self.value.append(str(self.combo.currentText()))
        self.value.append(str(self.editor.text()))
        self.set_value(self.value)
        self.label.setFocus()
        # emiting editingFinished without a value for the mechanism itself will lead to
        # integrity errors
        if self.value[1]:
            self.emit(editingFinished)



import elixir
from elixir import *
from sqlalchemy import *
from sqlalchemy.orm import scoped_session, create_session

elixir.session = scoped_session( create_session )

import settings

metadata = MetaData()

__metadata__ = metadata

__metadata__.bind = settings.ENGINE()
__metadata__.autoflush = False
__metadata__.transactional = False

from authentication import *
from fixture import *
from i18n import *
from memento import *
from synchronization import *
from type_and_status import *



'''
Created on Sep 12, 2009

@author: tw55413
'''

from signal_slot_model_thread import AbstractModelThread, Task, setup_model

class NoThreadModelThread(AbstractModelThread):

    def __init__(self, setup_thread = setup_model ):
        self.responses = []
        AbstractModelThread.__init__(self, setup_thread = setup_model )
        self._setup_thread()

    def start(self):
        pass

    def post( self, request, response = lambda result:None,
             exception = lambda exc:None ):
        task = Task(request)
        if response:
            task.connect(task, task.finished, response)
        if exception:
            task.connect(task, task.exception, exception)
        task.execute()

    def isRunning(self):
        return True



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""Class decorator to make all fields visualized with the Admin into read-only
fields"""

def notEditableAdmin(original_admin):
    """Turn all fields visualized with original_admin into read only fields
  :param original_admin: an implementation of ObjectAdmin

  usage ::

    class Movie(Entity):
      name = Field(Unicode(50))

      class Admin(EntityAdmin):
        list_display = ['name']

      Admin = notEditableAdmin(Admin)
    """

    class NewAdmin(original_admin):

#    def get_related_entity_admin(self, entity):
#      admin = original_admin.get_related_entity_admin(self, entity)
#      return notEditableAdmin(admin)

        def get_field_attributes(self, field_name):
            attribs = original_admin.get_field_attributes(self, field_name)
            attribs['editable'] = False
            return attribs

    return NewAdmin



import logging
logger = logging.getLogger('camelot.view.export.html')

def open_html_in_desktop_service(html):
    import os
    import tempfile
    html_fd, html_fn = tempfile.mkstemp(suffix='.html')
    html_file = os.fdopen(html_fd, 'wb')
    html_file.write(html.encode('utf-8'))
    html_file.close()
    from PyQt4 import QtGui, QtCore
    QtGui.QDesktopServices.openUrl(QtCore.QUrl('file://%s' % html_fn))



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""
Camelot extends the SQLAlchemy field types with a number of its own field
types. Those field types are automatically mapped to a specific delegate taking
care of the visualisation.

Those fields are stored in the :mod:`camelot.types` module.
"""

import logging
logger = logging.getLogger('camelot.types')

from sqlalchemy import types

from camelot.core.files.storage import StoredFile, StoredImage, Storage

class VirtualAddress(types.TypeDecorator):
    """A single field that can be used to enter phone numbers, fax numbers, email
    addresses, im addresses.  The editor provides soft validation of the data
    entered.  The address or number is stored as a string in the database.
      
    This column type accepts and returns tuples of strings, the first string is
    the :attr:`virtual_address_type`, and the second the address itself.
  
    eg: ``('email','project-camelot@conceptive.be')`` is stored as
    ``mail://project-camelot@conceptive.be``
  
    .. image:: ../_static/virtualaddress_editor.png
    """
    
    impl = types.Unicode
    virtual_address_types = ['phone', 'fax', 'mobile', 'email', 'im', 'pager',]
  
    def bind_processor(self, dialect):
  
        impl_processor = self.impl.bind_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
          
        def processor(value):
            if value is not None:
                if value[1]:
                    value = u'://'.join(value)
                else:
                    value = None
            return impl_processor(value)
          
        return processor
    
    def result_processor(self, dialect):
      
        impl_processor = self.impl.result_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
      
        def processor(value):
    
            if value:
                split = value.split('://')
                if len(split)>1:
                    return tuple(split)
            return (u'phone',u'')
            
        return processor  
        
    
class Code(types.TypeDecorator):
    """SQLAlchemy column type to store codes.  Where a code is a list of strings
    on which a regular expression can be enforced.
  
    This column type accepts and returns a list of strings and stores them as a
    string joined with points.
  
    eg: ``['08', 'AB']`` is stored as ``08.AB``
    """
    
    impl = types.Unicode
          
    def __init__(self, parts, **kwargs):
        """
        :param parts: a list of input masks specifying the mask for each part,
        eg ``['99', 'AA']``. For valid input masks, see
        `QLineEdit <http://www.riverbankcomputing.co.uk/static/Docs/PyQt4/html/qlineedit.html>`_
        """
        import string
        translator = string.maketrans('', '')
        self.parts = parts
        max_length = sum(len(part.translate(translator, '<>!')) for part in parts) + len(parts)
        types.TypeDecorator.__init__(self, length=max_length, **kwargs)
        
    def bind_processor(self, dialect):
  
        impl_processor = self.impl.bind_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
          
        def processor(value):
            if value is not None:
                value = u'.'.join(value)
            return impl_processor(value)
          
        return processor
    
    def result_processor(self, dialect):
      
        impl_processor = self.impl.result_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
      
        def processor(value):
    
            if value:
                return value.split('.')
            return ['' for _p in self.parts]
            
        return processor
    
class IPAddress(Code):
    def __init__(self, **kwargs):
        super(IPAddress, self).__init__(parts=['900','900','900','900'])
    
class Rating(types.TypeDecorator):
    """The rating field is an integer field that is visualized as a number of stars that
  can be selected::
  
    class Movie(Entity):
      title = Field(Unicode(60), required=True)
      rating = Field(camelot.types.Rating())
      
  .. image:: ../_static/rating.png"""
    
    impl = types.Integer
       
class RichText(types.TypeDecorator):
    """RichText fields are unlimited text fields which contain html.  The html will be
  rendered in a rich text editor.  
  
  .. image:: ../_static/richtext.png"""
    
    impl = types.UnicodeText
     
class Language(types.TypeDecorator):
    """The languages are stored as ISO codes in the database
    """
    
    import babel
    
    impl = types.Unicode
    choices = []
    for code in babel.localedata.list():
        locale = babel.Locale(code)
        display_name = locale.get_display_name()
        if display_name:
            choices.append((code, u'%s (%s)'%(code, display_name)))
        else:
            choices.append((code, code))
    
    def __init__(self):
        types.TypeDecorator.__init__(self, length=20)
        
class Color(types.TypeDecorator):
    """The Color field returns and accepts tuples of the form (r,g,b,a) where
  r,g,b,a are integers between 0 and 255. The color is stored as an hexadecimal
  string of the form AARRGGBB into the database, where AA is the transparency, 
  RR is red, GG is green BB is blue::
  
    class MovieType(Entity):
      color = Field(camelot.types.Color())
  
  .. image:: ../_static/color.png
  
  
  The colors are stored in the database as strings 
    """
    
    impl = types.Unicode
    
    def __init__(self):
        types.TypeDecorator.__init__(self, length=8)
        
    def bind_processor(self, dialect):
  
        impl_processor = self.impl.bind_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
          
        def processor(value):
            if value is not None:
                assert len(value) == 4
                for i in range(4):
                    assert value[i] >= 0
                    assert value[i] <= 255
                return '%02X%02X%02X%02X'%(value[3], value[0], value[1], value[2])
            return impl_processor(value)
          
        return processor
      
    def result_processor(self, dialect):
      
        impl_processor = self.impl.result_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
            
        def processor(value):
    
            if value:
                return (int(value[2:4],16), int(value[4:6],16), int(value[6:8],16), int(value[0:2],16))
              
        return processor
      
class Enumeration(types.TypeDecorator):
    """The enumeration field stores integers in the database, but represents them as
  strings.  This allows efficient storage and querying while preserving readable code.
  
  Typical use of this field would be a status field.
  
  Enumeration fields are visualized as a combo box, where the labels in the combo
  box are the capitalized strings::
  
    class Movie(Entity):
      title = Field(Unicode(60), required=True)
      state = Field(camelot.types.Enumeration([(1,'planned'), (2,'recording'), (3,'finished'), (4,'canceled')]), 
                                              index=True, required=True, default='planning')
  
  .. image:: ../_static/enumeration.png  
  """
    
    impl = types.Integer
    
    def __init__(self, choices=[], **kwargs):
        """
        @param param: choices is a list of tuples.  each tuple contains an integer and its
        associated string.  eg. : choices = [(1,'draft'), (2,'approved')]
        """
        types.TypeDecorator.__init__(self, **kwargs)
        self._int_to_string = dict(choices)
        self._string_to_int = dict((v,k) for (k,v) in choices)
        self.choices = [v for (k,v) in choices]
        
    def bind_processor(self, dialect):
  
        impl_processor = self.impl.bind_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
          
        def processor(value):
            if value is not None:
                try:
                    value = self._string_to_int[value]
                    return impl_processor(value)
                except KeyError, e:
                    logger.error('could not process enumeration value %s, possible values are %s'%(value, u', '.join(list(self._string_to_int.keys()))), exc_info=e)
                        
        return processor
    
    def result_processor(self, dialect):
      
        impl_processor = self.impl.result_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
            
        def processor(value):
            if value is not None:
                value = impl_processor(value)
                try:
                    return self._int_to_string[value]
                except KeyError, e:
                    logger.error('could not process %s'%value, exc_info=e)
                
        return processor
    
class File(types.TypeDecorator):
    """Sqlalchemy column type to store files.  Only the location of the file is stored
    
  This column type accepts and returns a StoredFile, and stores them in the directory
  specified by settings.MEDIA_ROOT.  The name of the file is stored as a string in
  the database.  A subdirectory upload_to can be specified::
  
    class Movie(Entity):
      script = Field(camelot.types.File(upload_to='script'))
      
  .. image:: ../_static/file_delegate.png
    """
    
    impl = types.Unicode
    stored_file_implementation = StoredFile
    
    def __init__(self, max_length=100, upload_to='', storage=Storage, **kwargs):
        self.max_length = max_length
        self.storage = storage(upload_to, self.stored_file_implementation)
        types.TypeDecorator.__init__(self, length=max_length, **kwargs)
        
    def bind_processor(self, dialect):
  
        impl_processor = self.impl.bind_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
          
        def processor(value):
            if value is not None:
                assert isinstance(value, (self.stored_file_implementation))
                return impl_processor(value.name)
            return impl_processor(value)
          
        return processor
    
    def result_processor(self, dialect):
      
        impl_processor = self.impl.result_processor(dialect)
        if not impl_processor:
            impl_processor = lambda x:x
            
        def processor(value):
    
            if value:
                value = impl_processor(value)
                return self.stored_file_implementation(self.storage, value)
              
        return processor
      
class Image(File):
    """Sqlalchemy column type to store images
    
  This column type accepts and returns a StoredImage, and stores them in the directory
  specified by settings.MEDIA_ROOT.  The name of the file is stored as a string in
  the database.
  
  The Image field type provides the same functionallity as the File field type, but
  the files stored should be images.
  
  .. image:: ../_static/image.png  
    """
  
    stored_file_implementation = StoredImage



from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from customdelegate import CustomDelegate, DocumentationMetaclass
from camelot.view.controls import editors
from camelot.core.utils import variant_to_pyobject
from camelot.view.proxy import ValueLoading

class PlainTextDelegate(CustomDelegate):
    """Custom delegate for simple string values"""
  
    __metaclass__ = DocumentationMetaclass
  
    editor = editors.TextLineEditor
      
    def __init__(self, parent=None, length=20, editable=True, **kwargs):
        CustomDelegate.__init__(self, parent, editable, length=length, **kwargs)
        self.editable = editable
        self.length = length
    
    def paint(self, painter, option, index):
        painter.save()
        self.drawBackground(painter, option, index)
        text = variant_to_pyobject(index.model().data(index, Qt.EditRole)) 
        
        background_color = QtGui.QColor(index.model().data(index, Qt.BackgroundRole))  
    
        rect = option.rect
        rect = QtCore.QRect(rect.left(), rect.top(), rect.width(), rect.height())  
    
        if( option.state & QtGui.QStyle.State_Selected ):
            painter.fillRect(option.rect, option.palette.highlight())
            fontColor = QtGui.QColor()
            if self.editable:         
                Color = option.palette.highlightedText().color()
                fontColor.setRgb(Color.red(), Color.green(), Color.blue())
            else:          
                fontColor.setRgb(130,130,130)
        else:
            if self.editable:
                painter.fillRect(option.rect, background_color)
                fontColor = QtGui.QColor()
                fontColor.setRgb(0,0,0)
            else:
                painter.fillRect(option.rect, option.palette.window())
                fontColor = QtGui.QColor()
                fontColor.setRgb(130,130,130)
      
        if text!=ValueLoading:
            text = text or ''
        else:
            text = ''
            
        painter.setPen(fontColor.toRgb())
        rect = QtCore.QRect(option.rect.left(),
                            option.rect.top(),
                            option.rect.width(),
                            option.rect.height())
        painter.drawText(rect.x() + 2,
                         rect.y(),
                         rect.width() - 4,
                         rect.height(),
                         Qt.AlignVCenter | Qt.AlignLeft,
                         unicode(text))
        painter.restore()



import logging
logger = logging.getLogger('camelot.view.export.outlook')

"""Functions to send files by email using outlook

After http://win32com.goermezer.de/content/view/227/192/
"""

def open_html_in_outlook(html):
  
    try:
        import pythoncom
        import win32com.client
        pythoncom.CoInitialize()
        outlook_app = win32com.client.Dispatch("Outlook.Application")
    except Exception, e:
        """We're probably not running windows"""
        logger.warn('unable to launch Outlook', exc_info=e)
        return
      
    msg = outlook_app.CreateItem(0)
    #msg.BodyFormat=2
    msg.HTMLBody=html
    #msg.Subject=o_subject
    msg.Display(True)



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from customeditor import CustomEditor
from camelot.core import constants


class BoolEditor(CustomEditor):
    """Widget for editing a boolean field"""
  
    def __init__(self,
                 parent=None,
                 minimum=constants.camelot_minint,
                 maximum=constants.camelot_maxint,
                 editable=True,
                 **kwargs):
        CustomEditor.__init__(self, parent)
        self.checkBox = QtGui.QCheckBox()
        self.checkBox.setEnabled(editable)
    
        layout = QtGui.QHBoxLayout()
        layout.setMargin(0)
        layout.setSpacing(0)
        layout.addWidget(self.checkBox)
        self.setFocusProxy(self.checkBox)
        self.setLayout(layout)


    def set_value(self, value):
        value = CustomEditor.set_value(self, value)
        if value:
            self.checkBox.setCheckState(Qt.Checked)
        else:
            self.checkBox.setCheckState(Qt.Unchecked)


    def get_value(self):
        value = self.checkBox.isChecked()
        return CustomEditor.get_value(self) or value


    def editingFinished(self, value=None):
        if value == None:
            value = self.checkBox.isChecked()
        self.emit(QtCore.SIGNAL('editingFinished()'), value)

      
    def set_enabled(self, editable=True):
        value = self.get_value()
        self.checkBox.setDisabled(not editable)
        self.set_value(value)


    def sizeHint(self):
        size = QtGui.QComboBox().sizeHint()
        return size



"""This module provides a singleton workspace that can be used by views
and widget to create new windows or raise existing ones"""

from PyQt4 import QtGui, QtCore

import logging

from camelot.view.model_thread import gui_function

logger = logging.getLogger('camelot.view.workspace')

class DesktopWorkspace(QtGui.QMdiArea):
  
    @gui_function
    def __init__(self, *args):
        QtGui.QMdiArea.__init__(self, *args)
        self.setOption(QtGui.QMdiArea.DontMaximizeSubWindowOnActivation)
        self.setBackground(QtGui.QBrush(QtGui.QColor('white')))
        self.setActivationOrder(QtGui.QMdiArea.ActivationHistoryOrder)
    
    @gui_function
    def addSubWindow(self, widget, *args):
        from camelot.view.controls.view import AbstractView
        subwindow = QtGui.QMdiArea.addSubWindow(self, widget, *args)
        if hasattr(widget, 'closeAfterValidation'):
            subwindow.connect(widget, widget.closeAfterValidation, subwindow, QtCore.SLOT("close()"))
            
        def create_set_window_title(subwindow):
          
            def set_window_title(new_title):
                subwindow.setWindowTitle(new_title)
                
            return set_window_title
            
        self.connect(widget, AbstractView.title_changed_signal, create_set_window_title(subwindow))
        return subwindow
    
class NoDesktopWorkspace(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self._windowlist = []
    
    @gui_function
    def addSubWindow(self, widget, *args):
        self.widget = widget
        self.widget.setParent(None)
        self.widget.show()
        self._windowlist.append(self.widget)
        self.connect(widget, QtCore.SIGNAL('WidgetClosed()'), self.removeWidgetFromWorkspace)
        
    @gui_function
    def subWindowList(self):
        return self._windowlist
      
    @gui_function
    def removeWidgetFromWorkspace(self):
        self._windowlist.remove(self.widget)
        
_workspace_ = []
        
@gui_function
def construct_workspace(*args, **kwargs):
    _workspace_.append(DesktopWorkspace())
    return _workspace_[0]
    
@gui_function
def construct_no_desktop_workspace(*args, **kwargs):
    _workspace_.append(NoDesktopWorkspace())
    return _workspace_[0]
  
@gui_function
def get_workspace():
    return _workspace_[0]



#  ===========================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ===========================================================================

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt, SIGNAL
from PyQt4.QtGui import QItemDelegate
from camelot.core.utils import variant_to_pyobject

from camelot.view.controls import editors
from camelot.core.utils import create_constant_function
from camelot.view.proxy import ValueLoading

# custom color
not_editable_background = QtGui.QColor(235, 233, 237)
# darkgray
not_editable_foreground = QtGui.QColor(Qt.darkGray)


def DocumentationMetaclass(name, bases, dct):
    dct['__doc__'] = dct.get('__doc__','') + """

.. image:: ../_static/delegates/%s_unselected_disabled.png
.. image:: ../_static/delegates/%s_unselected_editable.png
.. image:: ../_static/delegates/%s_selected_disabled.png
.. image:: ../_static/delegates/%s_selected_editable.png
"""%(name, name, name, name)
    return type(name, bases, dct)

  
class CustomDelegate(QItemDelegate):
    """Base class for implementing custom delegates.

.. attribute:: editor 

class attribute specifies the editor class that should be used
"""

    editor = None
  
    def __init__(self, parent=None, editable=True, **kwargs):
        """:param parent: the parent object for the delegate
:param editable: a boolean indicating if the field associated to the delegate
is editable"""
        QItemDelegate.__init__(self, parent)
        self.editable = editable
        self.kwargs = kwargs
        self._font_metrics = QtGui.QFontMetrics(QtGui.QApplication.font())
        self._height = self._font_metrics.lineSpacing() + 10
        self._width = self._font_metrics.averageCharWidth() * 20
    
    def createEditor(self, parent, option, index):
        """:param option: use an option with version 5 to indicate the widget
will be put onto a form"""
        editor = self.editor(parent, editable=self.editable, **self.kwargs)
        assert editor
        assert isinstance(editor, (QtGui.QWidget,))        
        if option.version != 5:
            editor.setAutoFillBackground(True)
        self.connect(editor,
                     editors.editingFinished,
                     self.commitAndCloseEditor)
        return editor
  
    def sizeHint(self, option, index):
        return QtCore.QSize(self._width, self._height)

    def commitAndCloseEditor(self):
        editor = self.sender()
        assert editor
        assert isinstance(editor, (QtGui.QWidget,))
        self.emit(SIGNAL('commitData(QWidget*)'), editor)
        sig = SIGNAL('closeEditor(QWidget*, \
                                  QAbstractItemDelegate::EndEditHint)')
        self.emit(sig, editor, QtGui.QAbstractItemDelegate.NoHint)

    def setEditorData(self, editor, index):
        value = variant_to_pyobject(index.model().data(index, Qt.EditRole))
        editor.set_value(value)
        index.model().data(index, Qt.ToolTipRole)
        tip = variant_to_pyobject(index.model().data(index, Qt.ToolTipRole))
        if tip not in (None, ValueLoading):
            editor.setToolTip(unicode(tip))
        else:
            editor.setToolTip('')

    def setModelData(self, editor, model, index):
        if isinstance(model, QtGui.QStandardItemModel):
            val = QtCore.QVariant(editor.get_value())
        else:
            val = create_constant_function(editor.get_value())
        model.setData(index, val)
        
    def paint_text(self, painter, option, index, text, margin_left=0, margin_right=0):
        """Paint unicode text into the given rect defined by option, and fill the rect with
        the background color
        :arg margin_left: additional margin to the left, to be used for icons or others
        :arg margin_right: additional margin to the right, to be used for icons or others"""
        
        background_color = QtGui.QColor(index.model().data(index, Qt.BackgroundRole))
        rect = option.rect
        
        if( option.state & QtGui.QStyle.State_Selected ):
            painter.fillRect(option.rect, option.palette.highlight())
            fontColor = QtGui.QColor()
            if self.editable:         
                Color = option.palette.highlightedText().color()
                fontColor.setRgb(Color.red(), Color.green(), Color.blue())
            else:          
                fontColor.setRgb(130,130,130)
        else:
            if self.editable:
                painter.fillRect(rect, background_color)
                fontColor = QtGui.QColor()
                fontColor.setRgb(0,0,0)
            else:
                painter.fillRect(rect, option.palette.window())
                fontColor = QtGui.QColor()
                fontColor.setRgb(130,130,130)
              
              
        painter.setPen(fontColor.toRgb())
        painter.drawText(rect.x() + 2 + margin_left,
                         rect.y(),
                         rect.width() - 4 - (margin_left + margin_right),
                         rect.height(),
                         Qt.AlignVCenter | Qt.AlignLeft,
                         text)        



import logging
from camelot.view import art

logger = logging.getLogger('camelot.view.export.word')

def open_html_in_word(html, template=art.file_('empty_document.doc'),
                      pre_processor=lambda doc:None,
                      post_processor=lambda doc:None):
    """Open MS Word through COM objects and import the specified html
    into a new document.
    @param html: the html to import
    @param template: the empty word document in which to import the html
    @param post_processor: a function that will be called before importing the
    html, with as its argument the COM Document.     
    @param post_processor: a function that will be called after importing the
    html, with as its argument the COM Document.   
    """
    import tempfile
    import os, sys
    
    html_fd, html_fn = tempfile.mkstemp(suffix='.html')
    html_file = os.fdopen(html_fd, 'wb')
    html_file.write(html.encode('utf-8'))
    html_file.close()
  
    if 'win' in sys.platform:
        try:
            import pythoncom
            import win32com.client
            pythoncom.CoInitialize()
            word_app = win32com.client.Dispatch("Word.Application")
            doc_fd, doc_fn = tempfile.mkstemp(suffix='.doc')
            os.close(doc_fd)
            word_app.Visible = True
            doc = word_app.Documents.Open(template)
            word_app.ActiveDocument.SaveAs(doc_fn)
            section = doc.Sections(1)
            pre_processor(doc)
            section.Range.InsertFile(FileName=html_fn)
            post_processor(doc)
            doc.Activate()
            word_app.Activate()
        except Exception, e:
            logger.warn('unable to launch word', exc_info=e)
    else:
        """We're probably not running windows, so let OS handle it (used to be abiword)"""
        from PyQt4 import QtGui, QtCore
        QtGui.QDesktopServices.openUrl(QtCore.QUrl('file://%s' % html_fn)) 
        return
    



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

import logging
from functools import wraps

from PyQt4 import QtCore, QtGui

logger = logging.getLogger( 'camelot.view.model_thread' )

_model_thread_ = []

class ModelThreadException( Exception ):
    pass

def model_function( original_function ):
    """Decorator to ensure a function is only called from within the model
    thread.  If this function is called in another thread, an exception will be
    thrown
    """

    def in_model_thread():
        """return wether current thread is model thread"""
        from no_thread_model_thread import NoThreadModelThread
        current_thread = QtCore.QThread.currentThread()
        model_thread = get_model_thread()
        return (current_thread==model_thread) or isinstance(model_thread, (NoThreadModelThread,))

    @wraps(original_function)
    def wrapper( *args, **kwargs ):
        assert in_model_thread()
        return original_function( *args, **kwargs )

    return wrapper

def gui_function( original_function ):
    """Decorator to ensure a function is only called from within the gui thread.
    If this function is called in another thread, an exception will be thrown
    """

    def in_gui_thread():
        """return wether current thread is gui thread"""
        gui_thread = QtGui.QApplication.instance().thread()
        current_thread = QtCore.QThread.currentThread()
        return gui_thread == current_thread

    @wraps( original_function )
    def wrapper( *args, **kwargs ):
        assert in_gui_thread()
        return original_function( *args, **kwargs )

    return wrapper

def setup_model():
    """Call the setup_model function in the settings"""
    from settings import setup_model
    setup_model()

class AbstractModelThread(object):
    """Abstract implementation of a model thread class
  Thread in which the model runs, all requests to the model should be
  posted to the the model thread.

  This class ensures the gui thread doesn't block when the model needs
  time to complete tasks by providing asynchronous communication between
  the model thread and the gui thread
  """

    thread_busy_signal = QtCore.SIGNAL('thread_busy')
    
    def __init__(self, setup_thread = setup_model ):
        """
        @param setup_thread: function to be called at startup of the thread to initialize
        everything, by default this will setup the model.  set to None if nothing should
        be done.
        """
        self.logger = logging.getLogger( logger.name + '.%s' % id( self ) )
        self._setup_thread = setup_thread
        self._exit = False
        self._traceback = ''
        self.logger.debug( 'model thread constructed' )

    def run(self):
        pass

    def traceback( self ):
        """The formatted traceback of the last exception in the model thread"""
        return self._traceback

    def wait_on_work(self):
        """Wait for all work to be finished, this function should only be used
    to do unit testing and such, since it will block the calling thread until
    all work is done"""
        pass

    def process_responses(self):
        pass

    def post_response( self, response, arg ):
        raise NotImplemented

    def post( self, request, response = None, exception = None ):
        """Post a request to the model thread, request should be
    a function that takes no arguments.  The request function
    will be called within the model thread.  When the request
    is finished, on first occasion, the response function will be
    called within the gui thread.  The response function takes as
    arguments, the results of the request function.
    @param request: function to be called within the model thread
    @param response: a slot that will be called with the result of the
    request function
    @param exception: a slot that will be called in case request throws
    an exception
        """
        raise NotImplemented

def construct_model_thread( *args, **kwargs ):
    from signal_slot_model_thread import SignalSlotModelThread
    _model_thread_.insert( 0, SignalSlotModelThread( *args, **kwargs ) )

def has_model_thread():
    return len( _model_thread_ ) > 0

def get_model_thread():
    return _model_thread_[0]

def post(request, response = None, exception = None):
    """Post a request and a response to the default model thread"""
    mt = get_model_thread()
    mt.post(request, response, exception)



from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from customdelegate import CustomDelegate, DocumentationMetaclass
from camelot.view.controls import editors
from camelot.core.utils import variant_to_pyobject
from camelot.view.proxy import ValueLoading

class CodeDelegate(CustomDelegate):
  
    __metaclass__ = DocumentationMetaclass
      
    editor = editors.CodeEditor
    
    def __init__(self, parent=None, parts=[], **kwargs):
        CustomDelegate.__init__(self, parent=parent, parts=parts, **kwargs)
        self.parts = parts
    
    def paint(self, painter, option, index):
        painter.save()
        numParts = len(self.parts)
        self.drawBackground(painter, option, index)
        
        background_color = QtGui.QColor(index.model().data(index, Qt.BackgroundRole))
        
        if( option.state & QtGui.QStyle.State_Selected ):
            painter.fillRect(option.rect, option.palette.highlight())
            fontColor = QtGui.QColor()
            if self.editable:
                Color = option.palette.highlightedText().color()
                fontColor.setRgb(Color.red(), Color.green(), Color.blue())
            else:
                fontColor.setRgb(130,130,130)
        else:
            if self.editable:
                painter.fillRect(option.rect, background_color)
                fontColor = QtGui.QColor()
                fontColor.setRgb(0,0,0)
            else:
                painter.fillRect(option.rect, option.palette.window())
                fontColor = QtGui.QColor()
                fontColor.setRgb(130,130,130)
                
          
        rect = option.rect
        rect = QtCore.QRect(rect.left()+3, rect.top()+6, rect.width(), rect.height()-3)
        
        
        if numParts != 0:
            value = variant_to_pyobject(index.model().data(index, Qt.EditRole)) or []
            if value == ValueLoading:
                value = []
            value = '.'.join([unicode(i) for i in value])
            
            
            painter.setPen(fontColor.toRgb())
            
            painter.drawText(rect.x(),
                           rect.y()-4,
                           rect.width()-6,
                           rect.height(),
                           Qt.AlignVCenter | Qt.AlignRight,
                           value)  
            
      
              
            
        
        painter.restore()
      
    
#  def sizeHint(self, option, index):
#    return self._dummy_editor.sizeHint() 



import logging
logger = logging.getLogger('camelot.view.controls.delegates.delegatemanager')

from PyQt4 import QtGui, QtCore

class DelegateManager(QtGui.QItemDelegate):
    """Manages custom delegates, should not be used by the application
  developer
  """
  
    def __init__(self, parent=None, **kwargs):
        QtGui.QItemDelegate.__init__(self, parent)
        self.delegates = {}
    
    def set_columns_desc(self, columnsdesc):
        self.columnsdesc = columnsdesc
        
    def get_column_delegate(self, column):
        try:
            return self.delegates[column]
        except KeyError:
            logger.error('Programming Error, no delegate available for column %s'%column)
            logger.error('Available columns : %s'%unicode(self.delegates.keys()))
            raise KeyError
    
    def insertColumnDelegate(self, column, delegate):
        """Inserts a custom column delegate"""
        logger.debug('inserting delegate for column %s' % column)
        assert delegate
        delegate.setParent(self)
        self.delegates[column] = delegate
        self.connect(delegate, QtCore.SIGNAL('commitData(QWidget*)'), self.commitData)
        self.connect(delegate, QtCore.SIGNAL('closeEditor(QWidget*)'), self.closeEditor)
    
    def commitData(self, editor):
        self.emit(QtCore.SIGNAL('commitData(QWidget*)'), editor)
        
    def closeEditor(self, editor):
        self.emit(QtCore.SIGNAL('closeEditor(QWidget*)'), editor)
        
    def removeColumnDelegate(self, column):
        """Removes custom column delegate"""
        logger.debug('removing a custom column delegate')
        if column in self.delegates:
            del self.delegates[column]
      
    def paint(self, painter, option, index):
        """Use a custom delegate paint method if it exists"""
        delegate = self.get_column_delegate(index.column())
        delegate.paint(painter, option, index)
      
    def createEditor(self, parent, option, index):
        """Use a custom delegate createEditor method if it exists"""
        delegate = self.get_column_delegate(index.column())
        editor = delegate.createEditor(parent, option, index)
        assert editor
        return editor
    
    def setEditorData(self, editor, index):
        """Use a custom delegate setEditorData method if it exists"""
        logger.debug('setting editor data for column %s' % index.column())
        delegate = self.get_column_delegate(index.column())
        delegate.setEditorData(editor, index)
      
    def setModelData(self, editor, model, index):
        """Use a custom delegate setModelData method if it exists"""
        logger.debug('setting model data for column %s' % index.column())
        delegate = self.get_column_delegate(index.column())
        delegate.setModelData(editor, model, index)
            
    def sizeHint(self, option, index):
        option = QtGui.QStyleOptionViewItem()
        delegate = self.get_column_delegate(index.column())
        return delegate.sizeHint(option, index)



from PyQt4 import QtGui
from PyQt4.QtCore import Qt

from customdelegate import DocumentationMetaclass
from camelot.core.utils import variant_to_pyobject
from camelot.view.proxy import ValueLoading

class IntervalsDelegate(QtGui.QItemDelegate):
    """Custom delegate for visualizing camelot.container.IntervalsContainer
  data
  """
  
    __metaclass__ = DocumentationMetaclass
  
    def __init__(self, parent=None, editable=False, **kwargs):
        QtGui.QItemDelegate.__init__(self, parent)
        self.editable = editable
    
    def paint(self, painter, option, index):
        painter.save()
        self.drawBackground(painter, option, index)
        intervals = variant_to_pyobject(index.model().data(index, Qt.EditRole))
        
        background_color = QtGui.QColor(index.model().data(index, Qt.BackgroundRole))
        
        if( option.state & QtGui.QStyle.State_Selected ):
            painter.fillRect(option.rect, option.palette.highlight())
        else:
            if not self.editable:
                painter.fillRect(option.rect, option.palette.window())
            else:
                painter.fillRect(option.rect, background_color)
          
          
          
        if intervals and intervals!=ValueLoading:
            rect = option.rect
            xscale = float(rect.width()-4)/(intervals.max-intervals.min)
            xoffset = intervals.min * xscale + rect.x()
            yoffset = rect.y() + rect.height()/2
            for interval in intervals.intervals:
                qcolor = QtGui.QColor()
                qcolor.setRgb(*interval.color)
                pen = QtGui.QPen(qcolor)
                pen.setWidth(3)
                painter.setPen(pen)
                x1, x2 =  xoffset + interval.begin*xscale, xoffset + interval.end*xscale
                painter.drawLine(x1, yoffset, x2, yoffset)
                painter.drawEllipse(x1-1, yoffset-1, 2, 2)
                painter.drawEllipse(x2-1, yoffset-1, 2, 2)
                pen = QtGui.QPen(Qt.white)      
                
        painter.restore()
    
    def createEditor(self, parent, option, index):
        pass
    
    def setEditorData(self, editor, index):
        pass
    
    def setModelData(self, editor, model, index):
        pass



import logging

logging.basicConfig(level=logging.INFO, format='[%(levelname)-7s] [%(name)-35s] - %(message)s')
#logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)

def setup_model():
  from camelot.model import *
  from camelot.model.memento import *
  from camelot.model.synchronization import *
  from camelot.model.authentication import *
  from camelot.model.i18n import *
  setup_all(create_tables=True)
  updateLastLogin()

CAMELOT_MEDIA_ROOT = 'media'

ENGINE = lambda:'sqlite:///'


from PyQt4 import QtGui, QtCore

from customeditor import AbstractCustomEditor
import sip

class ChoicesEditor(QtGui.QComboBox, AbstractCustomEditor):

    def __init__(self, parent=None, editable=True, **kwargs):
        QtGui.QComboBox.__init__(self, parent)
        AbstractCustomEditor.__init__(self)
        self.setEnabled(editable)
        
    def set_choices(self, choices):
        """
    :param choices: a list of (value,name) tuples.  name will be displayed in the combobox,
    while value will be used within get_value and set_value
        """
        if not sip.isdeleted(self):
            allready_in_combobox = dict(self.get_choices())
            items_to_remove = []
            for i,(value,name) in enumerate(choices):
                if value not in allready_in_combobox:
                    self.insertItem(i, unicode(name), QtCore.QVariant(value))
                else:
                    # the editor data might allready have been set, but its name is
                    # still ..., therefore we set the name now correct
                    self.setItemText(i, unicode(name))
            current_value = self.get_value()
            new_choices = dict(choices)
            for i,(value,name) in enumerate(self.get_choices()):
                if (value not in new_choices) and (value!=current_value):
                    items_to_remove.append(i)
            removed_items = 0
            for i in items_to_remove:
                self.removeItem(i-removed_items)
                removed_items += 1
        
        
        
    def set_enabled(self, editable=True):
        self.setEnabled(editable)
    
    
    
              
    def get_choices(self):
        """
    :rtype: a list of (value,name) tuples
    """
        from camelot.core.utils import variant_to_pyobject
        return [(variant_to_pyobject(self.itemData(i)),unicode(self.itemText(i))) for i in range(self.count())]
        
    def set_value(self, value):
        if not sip.isdeleted(self):
            from camelot.core.utils import variant_to_pyobject
            value = AbstractCustomEditor.set_value(self, value)
            if value not in (None, NotImplemented):
                for i in range(self.count()):
                    if value == variant_to_pyobject(self.itemData(i)):
                        self.setCurrentIndex(i)
                        return
                # it might happen, that when we set the editor data, the set_choices 
                # method has not happened yet, therefore, we temporary set ... in the
                # text while setting the correct data to the editor
                self.insertItem(self.count(), '...', QtCore.QVariant(value))
                self.setCurrentIndex(self.count()-1)
            
    def get_value(self):
        if not sip.isdeleted(self):
            from camelot.core.utils import variant_to_pyobject
            current_index = self.currentIndex()
            if current_index>=0:
                value = variant_to_pyobject(self.itemData(self.currentIndex()))
            else:
                value = None
            return AbstractCustomEditor.get_value(self) or value



"""
Created on Jul 19, 2009

@author: tw55413
"""

camelot_maxint = 2147483647
camelot_minint = -2147483648
camelot_maxfloat = 1e15
camelot_minfloat = -1e15
camelot_time_format = 'hh:mm'
strftime_time_format = '%H:%M'
camelot_datetime_format = 'dd-MM-yyyy hh:mm'
strftime_datetime_format = '%d-%m-%Y %H:%M'
camelot_date_format = 'dd-MM-yyyy'
strftime_date_format = '%d-%m-%Y'
camelot_small_icon_width = 16
camelot_small_icon_height = 16



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

import logging
logger = logging.getLogger('camelot.admin.validator.object_validator')

from camelot.view.fifo import fifo


class ObjectValidator(object):
    """A validator class for normal python objects.  By default this validator
    declares all objects valid.  Subclass this class and overwrite it's
    objectValidity method to change it's behaviour.
    """

    def __init__(self, admin, model):
        self.admin = admin
        self.model = model
        self.message_cache = fifo(10)

    def objectValidity(self, entity_instance):
        """:return: list of messages explaining invalid data
        empty list if object is valid
        """
        return []

    def isValid(self, row):
        """Verify if a row in a model is 'valid' meaning it could be flushed to
        the database
        """
        messages = []
        logger.debug('isValid for row %s' % row)
        try:
            entity_instance = self.model._get_object(row)
            if entity_instance:
                messages = self.objectValidity(entity_instance)
                self.message_cache.add_data(row, entity_instance.id, messages)
        except Exception, e:
            logger.error(
                'programming error while validating object',
                exc_info=e
            )
        valid = (len(messages) == 0)
        logger.debug('valid : %s' % valid)
        return valid

    def validityMessages(self, row):
        try:
            return self.message_cache.get_data_at_row(row)
        except KeyError:
            raise Exception(
                'Programming error : isValid should be called '
                'before calling validityMessage'
            )

    def validityDialog(self, row, parent):
        """Return a QDialog that asks the user to discard his changes or
        continue to edit the row until it is valid.
        """
        from PyQt4 import QtGui
        return QtGui.QMessageBox(
            QtGui.QMessageBox.Warning,
            'Invalid form',
            '\n'.join(self.validityMessages(row)),
            QtGui.QMessageBox.Ok | QtGui.QMessageBox.Discard,
            parent
        )



from PyQt4 import QtGui

from customeditor import AbstractCustomEditor

class TextLineEditor(QtGui.QLineEdit, AbstractCustomEditor):

    def __init__(self, parent, length=20, editable=True, **kwargs):
        QtGui.QLineEdit.__init__(self, parent)
        AbstractCustomEditor.__init__(self)
        if length:
            self.setMaxLength(length)
        if not editable:
            self.setEnabled(False)
      
    def set_value(self, value):
        value = AbstractCustomEditor.set_value(self, value)
        if value:
            self.setText(unicode(value))
        else:
            self.setText('')
      
    def get_value(self):
        return AbstractCustomEditor.get_value(self) or unicode(self.text())
      
    def set_enabled(self, editable=True):
        value = self.text()
        self.setEnabled(editable)
        self.setText(value)



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""form view"""

import logging
logger = logging.getLogger( 'camelot.view.controls.formview' )

from PyQt4 import QtCore
from PyQt4 import QtGui
from camelot.view.model_thread import model_function, post
from camelot.view.controls.view import AbstractView

class FormView( AbstractView ):
    def __init__( self, title, admin, model, index ):
        AbstractView.__init__( self )
        self.title_prefix = title
        self.admin = admin
        self.model = model
        self.index = index
        self.change_title(title)
        self.widget_mapper = QtGui.QDataWidgetMapper()
        self.widget_layout = QtGui.QHBoxLayout()
    
        self.closeAfterValidation = QtCore.SIGNAL( 'closeAfterValidation()' )
        sig = 'dataChanged(const QModelIndex &, const QModelIndex &)'
        self.connect( self.model, QtCore.SIGNAL( sig ), self.dataChanged )
        self.connect( self.model, self.model.item_delegate_changed_signal, self.item_delegate_changed )
    
        self.widget_mapper.setModel( model )
        self.setLayout( self.widget_layout )
    
        if hasattr( admin, 'form_size' ) and admin.form_size:
            self.setMinimumSize( admin.form_size[0], admin.form_size[1] )
      
        self.validator = admin.create_validator( model )
        self.validate_before_close = True
        self.form = None
        self.columns = None
        self.delegate = None
    
        def getColumnsAndForm():
            return ( self.model.getColumns(), self.admin.get_form_display() )
      
        post( getColumnsAndForm, self.handleGetColumnsAndForm )
    
        def getActions():
            return admin.get_form_actions( None )
      
        post( getActions, self.setActions )
        self.update_title()
    
    def update_title( self ):
  
        def get_title():
            obj = self.getEntity()
            return u'%s %s' % ( self.title_prefix, self.admin.get_verbose_identifier( obj ) )
      
        post( get_title, self.change_title )
    
    def dataChanged( self, index_from, index_to ):
        #@TODO: only revert if this form is in the changed range
        self.widget_mapper.revert()
        self.update_title()
    
    def handleGetColumnsAndForm( self, columns_and_form ):
        self.columns, self.form = columns_and_form
        self.setColumnsFormAndDelegate()


    def item_delegate_changed(self):
        from camelot.view.controls.delegates.delegatemanager import DelegateManager
        self.delegate = self.model.getItemDelegate()
        assert self.delegate
        assert isinstance(self.delegate, DelegateManager)
        self.setColumnsFormAndDelegate()
    
    def setColumnsFormAndDelegate( self ):
        """Create value and label widgets"""
        # only if all information is available, we can start building the form
        if not (self.form and self.columns and self.delegate):
            return
        widgets = {}
        self.widget_mapper.setItemDelegate( self.delegate )
        option = QtGui.QStyleOptionViewItem()
        # set version to 5 to indicate the widget will appear on a
        # a form view and not on a table view
        option.version = 5
    
        for i, ( field_name, field_attributes ) in enumerate( self.columns ):
            model_index = self.model.index( self.index, i )
            hide_title = False
            if 'hide_title' in field_attributes:
                hide_title = field_attributes['hide_title']
            widget_label = None
            if not hide_title:
                widget_label = QtGui.QLabel( unicode(field_attributes['name']) )
            widget_editor = self.delegate.createEditor( self, option, model_index )
      
            # required fields font is bold
            if ( 'nullable' in field_attributes ) and \
               ( not field_attributes['nullable'] ):
                font = QtGui.QApplication.font()
                font.setBold( True )
                widget_label.setFont( font )
        
            assert widget_editor
            assert isinstance(widget_editor, QtGui.QWidget)
            
            self.widget_mapper.addMapping( widget_editor, i )
            widgets[field_name] = ( widget_label, widget_editor )
    
        self.widget_mapper.setCurrentIndex( self.index )
        self.widget_layout.insertWidget( 0, self.form.render( widgets, self ) )
        self.widget_layout.setContentsMargins( 7, 7, 7, 7 )
    
    def getEntity( self ):
        return self.model._get_object( self.widget_mapper.currentIndex() )
    
    def setActions( self, actions ):
        if actions:
            from actionsbox import ActionsBox
            logger.debug( 'setting Actions for formview' )
            self.actions_widget = ActionsBox( self, self.getEntity )
            self.actions_widget.setActions( actions )
            self.widget_layout.insertWidget( 1, self.actions_widget )
      
    def viewFirst( self ):
        """select model's first row"""
        # submit should not happen a second time, since then we don't want
        # the widgets data to be written to the model
        self.widget_mapper.submit()
        self.widget_mapper.toFirst()
        self.update_title()
    
    def viewLast( self ):
        """select model's last row"""
        # submit should not happen a second time, since then we don't want
        # the widgets data to be written to the model
        self.widget_mapper.submit()
        self.widget_mapper.toLast()
        self.update_title()
    
    def viewNext( self ):
        """select model's next row"""
        # submit should not happen a second time, since then we don't want
        # the widgets data to be written to the model
        self.widget_mapper.submit()
        self.widget_mapper.toNext()
        self.update_title()
    
    def viewPrevious( self ):
        """select model's previous row"""
        # submit should not happen a second time, since then we don't want
        # the widgets data to be written to the model
        self.widget_mapper.submit()
        self.widget_mapper.toPrevious()
        self.update_title()
    
    def showMessage( self, valid ):
        import sip
        if not valid:
            reply = self.validator.validityDialog( self.widget_mapper.currentIndex(), self ).exec_()
            if reply == QtGui.QMessageBox.Discard:
            # clear mapping to prevent data being written again to the model,
            # then we reverted the row
                self.widget_mapper.clearMapping()
                self.model.revertRow( self.widget_mapper.currentIndex() )
                self.validate_before_close = False
                self.emit( self.closeAfterValidation )
        else:
            self.validate_before_close = False
            if not sip.isdeleted( self ):
                self.emit( self.closeAfterValidation )
                    
    def validateClose( self ):
        logger.debug( 'validate before close : %s' % self.validate_before_close )
        if self.validate_before_close:
            # submit should not happen a second time, since then we don't
            # want the widgets data to be written to the model
            self.widget_mapper.submit()
      
            def validate():
                return self.validator.isValid( self.widget_mapper.currentIndex() )
        
            post( validate, self.showMessage )
            return False
      
        return True
    
    def closeEvent( self, event ):
        logger.debug( 'formview closed' )
        if self.validateClose():
            event.accept()
        else:
            event.ignore()
      
    @model_function
    def toHtml( self ):
        """generates html of the form"""
        from jinja import Environment
    
        def to_html( d = u'' ):
            """Jinja 1 filter to convert field values to their default html
            representation
            """
      
            def wrapped_in_table( env, context, value ):
                if isinstance( value, list ):
                    return u'<table><tr><td>' + \
                           u'</td></tr><tr><td>'.join( [unicode( e ) for e in value] ) + \
                           u'</td></tr></table>'
                return unicode( value )
        
            return wrapped_in_table
      
        entity = self.getEntity()
        fields = self.admin.get_fields()
        table = [dict( field_attributes = field_attributes,
                      value = getattr( entity, name ) )
                      for name, field_attributes in fields]
    
        context = {
          'title': self.admin.get_verbose_name(),
          'table': table,
        }
    
        from camelot.view.templates import loader
        env = Environment( loader = loader )
        env.filters['to_html'] = to_html
        tp = env.get_template( 'form_view.html' )
    
        return tp.render( context )



from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from customdelegate import CustomDelegate, DocumentationMetaclass
from camelot.view.controls import editors

class RichTextDelegate(CustomDelegate):
    """
  """
  
    __metaclass__ = DocumentationMetaclass
    
    editor = editors.RichTextEditor
    
    
    def __init__(self, parent=None, editable=True, **kwargs):
        CustomDelegate.__init__(self, parent, editable)
        self.editable = editable
        self._height = self._height * 10
        self._width = self._width * 3
    
    def paint(self, painter, option, index):
        painter.save()
        self.drawBackground(painter, option, index)
        unstrippedText = unicode(index.model().data(index, Qt.EditRole).toString())
        
        background_color = QtGui.QColor(index.model().data(index, Qt.BackgroundRole))
        
        if not unstrippedText:
            text = ''
        else:
    
            from HTMLParser import HTMLParser
      
            string = []
      
            class HtmlToTextParser(HTMLParser):
                def handle_data(self, data):
                    string.append(data.replace('\n',''))
          
            parser = HtmlToTextParser()
            parser.feed(unstrippedText)
      
            text = (' '.join(string))[:256]  
          
        rect = option.rect
        rect = QtCore.QRect(rect.left(), rect.top(), rect.width(), rect.height())  
          
        if( option.state & QtGui.QStyle.State_Selected ):
            painter.fillRect(option.rect, option.palette.highlight())
            fontColor = QtGui.QColor()
            if self.editable:
                Color = option.palette.highlightedText().color()
                fontColor.setRgb(Color.red(), Color.green(), Color.blue())
            else:
                fontColor.setRgb(130,130,130)
        else:
            if self.editable:
                painter.fillRect(option.rect, background_color)
                fontColor = QtGui.QColor()
                fontColor.setRgb(0,0,0)
            else:
                painter.fillRect(option.rect, option.palette.window())
                fontColor = QtGui.QColor()
                fontColor.setRgb(130,130,130)
              
              
        painter.setPen(fontColor.toRgb())
        rect = QtCore.QRect(option.rect.left(),
                            option.rect.top(),
                            option.rect.width(),
                            option.rect.height())
        painter.drawText(rect.x() + 2,
                         rect.y(),
                         rect.width() - 4,
                         rect.height(),
                         Qt.AlignVCenter | Qt.AlignLeft,
                         text)
        painter.restore()
    




class ApplicationAction(object):
    """An action that can be triggered by the user at the application level"""

    def run(self, parent):
        """Execute the action, called within the gui thread
        
:param parent: a QWidget that can be used as a parent for widgets during the 
execution of the action
    """
        raise NotImplemented()
    
    def get_verbose_name(self):
        """:return: the name of the action, as it can be shown to the user"""
        raise NotImplemented()

class TableViewAction(ApplicationAction):
    """An application action that opens a TableView for an Entity"""

    def __init__(self, entity, admin=None, verbose_name=None, parent_admin=None):
        from camelot.admin.application_admin import get_application_admin
        self.parent_admin = parent_admin or get_application_admin()
        if admin:
            self.admin = admin(self.parent_admin, entity)
        else:
            self.admin = self.parent_admin.get_entity_admin(entity)
        self.entity = entity
        self.verbose_name = verbose_name

    def get_verbose_name(self):
        return unicode(self.verbose_name or self.admin.get_verbose_name_plural())

    def run(self, parent):
        tableview = self.admin.create_table_view(parent)
        return tableview

def structure_to_application_action(structure):
    """Convert a python structure to an ApplicationAction"""
    if isinstance(structure, (ApplicationAction,)):
        return structure
    return TableViewAction(structure)



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""Manages icons and artworks"""

import os
import logging
logger = logging.getLogger('camelot.view.art')

from camelot.view.model_thread import gui_function


def file_(name):
    from pkg_resources import resource_filename
    import camelot
    return resource_filename(camelot.__name__, 'art/%s'%name)

class Pixmap(object):
    """Load pixmaps from the camelot art library"""

    def __init__(self, path, module=None):
        """:param path: the path of the pixmap relative to the art directory, use
    '/' as a path separator
    :param module: the module that contains the art directory, if None is given
    this will be camelot"""
        self._path = path
        if not module:
            import camelot
            self._module_name = camelot.__name__
        else:
            self._module_name = module.__name__

    def __unicode__(self):
        return self._path

    def __repr__(self):
        return self.__class__.__name__ + "('" + self._path + "')"

    def fullpath(self):
        """Obsolete : avoid this method, since it will copy the resource file
        from its package and copy it to a temp folder if the resource is
        packaged."""
        from pkg_resources import resource_filename
        pth = resource_filename(self._module_name, 'art/%s'%(self._path))
        if os.path.exists(pth):
            return pth
        else:
            return ''

    @gui_function
    def getQPixmap(self):
        """QPixmaps can only be used in the gui thread"""
        from pkg_resources import resource_string
        from PyQt4.QtGui import QPixmap
        qpm = QPixmap()
        success = qpm.loadFromData(resource_string(self._module_name,
                                                   'art/%s'%(self._path)))
        if not success:
            msg = u'Could not load pixmap %s from camelot art library'
            logger.warn(msg % self._path)
        return qpm

class Icon(Pixmap):
    """Manages paths to the icons images"""

    @gui_function
    def getQIcon(self):
        """QPixmaps can only be used in the gui thread"""
        from PyQt4.QtGui import QIcon
        return QIcon(self.getQPixmap())



import datetime

from PyQt4 import QtGui

from customeditor import AbstractCustomEditor
from camelot.core import constants

class TimeEditor(QtGui.QTimeEdit, AbstractCustomEditor):
  
    def __init__(self,
                 parent,
                 editable=True,
                 format=constants.camelot_time_format,
                 **kwargs):
        QtGui.QTimeEdit.__init__(self, parent)
        AbstractCustomEditor.__init__(self)
        self.setDisplayFormat(format)
        self.setEnabled(editable)
        
    def set_value(self, value):
        value = AbstractCustomEditor.set_value(self, value)
        if value:
            self.setTime(value)
        else:
            self.setTime(self.minimumTime())
            
    def get_value(self):
        value = self.time()
        value = datetime.time(hour=value.hour(),
                              minute=value.minute(),
                              second=value.second())
        return AbstractCustomEditor.get_value(self) or value
      
    def set_enabled(self, editable=True):
        self.setEnabled(editable)



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""
Helper functions to search through a collection of entities
"""
import logging

logger = logging.getLogger('camelot.view.search')

import camelot.types

def create_entity_search_query_decorator(admin, text):
    """create a query decorator to search through a collection of entities
    @param admin: the admin interface of the entity
    @param text: the text to search for
    @return: a function that can be applied to a query to make the query filter
    only the objects related to the requested text
    """
    from elixir import entities
    from sqlalchemy import orm

    if len(text.strip()):
        from sqlalchemy import Unicode, or_
        # arguments for the where clause
        args = []
        # join conditions : list of join entities
        joins = []
        search_tables = [admin.entity.table]
        for entity in entities:
            if issubclass(admin.entity, entity):
                search_tables.append(entity.table)

        def append_column(c):
            if issubclass(c.type.__class__, camelot.types.Color):
                pass
            elif issubclass(c.type.__class__, camelot.types.File):
                pass
            elif issubclass(c.type.__class__, camelot.types.Code):
                codes = text.split('.')
                args.append(c.like(['%'] + codes + ['%']))
                args.append(c.like(['%'] + codes))
                args.append(c.like(codes + ['%']))
            elif issubclass(c.type.__class__, camelot.types.VirtualAddress):
                args.append(c.like(('%','%'+text+'%')))
            elif issubclass(c.type.__class__, camelot.types.Image):
                pass
            elif issubclass(c.type.__class__, (Unicode, )) or \
                            (hasattr(c.type, 'impl') and \
                             issubclass(c.type.impl.__class__, (Unicode, ))):
                logger.debug('look in column : %s'%c.name)
                args.append(c.like('%'+text+'%'))

        for table in search_tables:
            for c in table._columns:
                append_column(c)

        for column_name in admin.list_search:
            path = column_name.split('.')
            target = admin.entity
            for p in path:
                mapper = orm.class_mapper(target)
                property = mapper.get_property(p, resolve_synonyms=True)
                if isinstance(property, orm.properties.PropertyLoader):
                    joins.append(getattr(target, p))
                    target = property._get_target().class_
                else:
                    append_column(property.columns[0])
                    #args.append(property.columns[0].like('%'+text+'%'))

        logger.debug('query joins : %s'%str(joins))
        logger.debug('query args : %s'%str(args))

        def create_query_decorator(joins, args):

            def query_decorator(q):
                for join in joins:
                    q = q.join(join)
                if len(args):
                    if len(args)>1:
                        q = q.filter(or_(*args))
                    else:
                        q = q.filter(args[0])
                return q

            return query_decorator

        return create_query_decorator(joins, args)



from one2manyeditor import One2ManyEditor, QtGui, QtCore
from customeditor import editingFinished
from abstractmanytooneeditor import AbstractManyToOneEditor

from camelot.view.art import Icon
from camelot.view.model_thread import model_function, post

class ManyToManyEditor( One2ManyEditor, AbstractManyToOneEditor ):

    def setupButtons( self, layout ):
        button_layout = QtGui.QVBoxLayout()
        button_layout.setSpacing( 0 )
        remove_button = QtGui.QToolButton()
        remove_button.setIcon( Icon( 'tango/16x16/actions/list-remove.png' ).getQIcon() )
        remove_button.setAutoRaise( True )
        remove_button.setFixedHeight( self.get_height() )
        self.connect( remove_button,
                     QtCore.SIGNAL( 'clicked()' ),
                     self.removeSelectedRows )
        add_button = QtGui.QToolButton()
        add_button.setIcon( Icon( 'tango/16x16/actions/list-add.png' ).getQIcon() )
        add_button.setAutoRaise( True )
        add_button.setFixedHeight( self.get_height() )
        self.connect( add_button, QtCore.SIGNAL( 'clicked()' ), self.createSelectView )
#    new_button = QtGui.QToolButton()
#    new_button.setIcon(Icon('tango/16x16/actions/document-new.png').getQIcon())
#    new_button.setAutoRaise(True)
#    self.connect(new_button, QtCore.SIGNAL('clicked()'), self.newRow)
        button_layout.addStretch()
        button_layout.addWidget( add_button )
        button_layout.addWidget( remove_button )
#    button_layout.addWidget(new_button)
        layout.addLayout( button_layout )
    
    def selectEntity( self, entity_instance_getter ):
  
        @model_function
        def insert():
            o = entity_instance_getter()
            self.model.insertEntityInstance( 0, o )
      
        post( insert, self.editingFinished )
      
    def editingFinished(self, *args):
        self.emit( editingFinished )
    
    def removeSelectedRows( self ):
        """Remove the selected rows in this tableview, but don't delete them"""
        for row in set( map( lambda x: x.row(), self.table.selectedIndexes() ) ):
            self.model.removeRow( row, delete = False )
        self.emit( editingFinished )



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""Module for managing imports"""

import logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('camelot.view.wizard.importwizard')

import csv
import codecs

from PyQt4.QtCore import Qt
from PyQt4 import QtCore, QtGui
from PyQt4.QtGui import QColor

from camelot.view.controls.editors.one2manyeditor import One2ManyEditor
from camelot.core.utils import ugettext as _
from camelot.view.art import Pixmap


class SelectFilePage(QtGui.QWizardPage):
    """SelectFilePage is the file selection page of the import wizard"""

    def __init__(self, parent=None):
        super(SelectFilePage, self).__init__(parent)
        self.setTitle(_('Import data from a file'))
        self.setSubTitle(_(
            "To import data, click 'Browse' to "
            "select a file then click 'Import'."
        ))

        icon = 'tango/32x32/mimetypes/x-office-spreadsheet.png'
        self.setPixmap(QtGui.QWizard.LogoPixmap, Pixmap(icon).getQPixmap())

        label = QtGui.QLabel(_('Select file:'))
        self.filelineedit = QtGui.QLineEdit()
        label.setBuddy(self.filelineedit)
        browsebutton = QtGui.QPushButton(_('Browse...'))

        # file path is a mandatory field
        self.registerField('datasource*', self.filelineedit)

        layout = QtGui.QVBoxLayout()
        layout.addWidget(label)
        hlayout = QtGui.QHBoxLayout()
        hlayout.addWidget(self.filelineedit)
        hlayout.addWidget(browsebutton)
        layout.addLayout(hlayout)
        self.setLayout(layout)

        self.connect(
            browsebutton,
            QtCore.SIGNAL('clicked()'),
            lambda: self.setpath()
        )

    def setpath(self):
        caption = _('Import Wizard - Set File Path')
        dir = self.field('datasource').toString()
        path = QtGui.QFileDialog.getOpenFileName(self, caption, dir)
        if path:
            self.filelineedit.setText(QtCore.QDir.toNativeSeparators(path))


class RowData(object):
    """Class representing the data in a single row of the imported file as an
    object with attributes column_1, column_2, ..., each representing the data
    in a single column of that row"""

    def __init__(self, row_number, row_data):
        """:param row_data: a list containing the data
        [column_1_data, column_2_data, ...] for a single row
        """
        self.id = row_number + 1
        for i, data in enumerate(row_data):
            self.__setattr__('column_%i' % i, data)


# see http://docs.python.org/library/csv.html
class UTF8Recoder:
    """Iterator that reads an encoded stream and reencodes the input to
    UTF-8."""

    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode('utf-8')


# see http://docs.python.org/library/csv.html
class UnicodeReader:
    """A CSV reader which will iterate over lines in the CSV file "f", which is
    encoded in the given encoding."""

    def __init__(self, f, dialect=csv.excel, encoding='utf-8', **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def next(self):
        row = self.reader.next()
        return [unicode(s, 'utf-8') for s in row]

    def __iter__(self):
        return self


class CsvCollectionGetter(object):
    """class that when called returns the data in filename as a list of RowData
    objects"""

    def __init__(self, filename):
        self.filename = filename
        self._data = None

    def __call__(self):
        if self._data==None:
            self._data = []
            import chardet
            
            enc = (
                chardet.detect(open(self.filename).read())['encoding']
                or 'utf-8'
            )
            items = UnicodeReader(open(self.filename), encoding=enc)

            self._data = [
                RowData(i, row_data)
                for i, row_data in enumerate(items)
            ]

        return self._data


class RowDataAdminDecorator(object):
    """Decorator that transforms the Admin of the class to be imported to an
    Admin of the RowData objects to be used when previewing and validating the
    data to be imported"""

    def __init__(self, object_admin):
        """:param object_admin: the object_admin object that will be
        decorated"""
        self._object_admin = object_admin

    def __getattr__(self, attr):
        return getattr(self._object_admin, attr)

    def get_columns(self):
        original_columns = self._object_admin.get_columns()

        def new_field_attributes(i, original_field_attributes, original_field):
            from camelot.view.controls import delegates
            attributes = dict(original_field_attributes)
            attributes['delegate'] = delegates.PlainTextDelegate
            attributes['python_type'] = str
            attributes['original_field'] = original_field

            if 'from_string' in attributes:

                def get_background_color(o):
                    """If the string is not convertible with from_string, or
                    the result is None when a value is required, set the
                    background to pink"""
                    value = getattr(o, 'column_%i'%i)
                    if not value and (attributes['nullable']==False):
                        return QColor('Pink')
                    try:
                        value = attributes['from_string'](value)
                        return None
                    except:
                        return QColor('Pink')

                attributes['background_color'] = get_background_color

            return attributes

        new_columns = [
            (
                'column_%i' %i,
                new_field_attributes(i, attributes, original_field)
            )
            for i, (original_field, attributes) in enumerate(original_columns)
        ]

        return new_columns


class DataPreviewPage(QtGui.QWizardPage):
    """DataPreviewPage is the previewing page for the import wizard"""

    def __init__(self, parent=None, model=None, collection_getter=None):
        super(DataPreviewPage, self).__init__(parent)
        assert model
        assert collection_getter
        self.setTitle(_('Data Preview'))
        self.setSubTitle(_('Please review the data below.'))
        self.model = model
        self.collection_getter = collection_getter

        icon = 'tango/32x32/mimetypes/x-office-spreadsheet.png'
        self.setPixmap(QtGui.QWizard.LogoPixmap, Pixmap(icon).getQPixmap())

        self.previewtable = One2ManyEditor(
            admin = model.get_admin(),
            parent=self,
            create_inline=True
        )

        ly = QtGui.QVBoxLayout()
        ly.addWidget(self.previewtable)
        self.setLayout(ly)

        self.setCommitPage(True)
        self.setButtonText(QtGui.QWizard.CommitButton, _('Import'))

    def initializePage(self):
        """Gets all info needed from SelectFilePage and feeds table"""
        filename = self.field('datasource').toString()
        self.model.set_collection_getter(self.collection_getter(filename))
        self.previewtable.set_value(self.model)
        self.emit(QtCore.SIGNAL('completeChanged()'))

    def isComplete(self):
        return True


class FinalPage(QtGui.QWizardPage):
    """FinalPage is the final page in the import process"""

    change_maximum_signal = QtCore.SIGNAL('change_maximum')
    change_value_signal = QtCore.SIGNAL('change_value')
    
    def __init__(self, parent=None, model=None):
        super(FinalPage, self).__init__(parent)
        self.setTitle(_('Import Progress'))
        self.model = model
        self.admin = model.get_admin()
        self.setSubTitle(_('Please wait while data is being imported.'))

        icon = 'tango/32x32/mimetypes/x-office-spreadsheet.png'
        self.setPixmap(QtGui.QWizard.LogoPixmap, Pixmap(icon).getQPixmap())
        self.setButtonText(QtGui.QWizard.FinishButton, _('Close'))
        self.progressbar = QtGui.QProgressBar()

        label = QtGui.QLabel(_(
            'The data will be ready when the progress reaches 100%.'
        ))
        label.setWordWrap(True)

        ly = QtGui.QVBoxLayout()
        ly.addWidget(label)
        ly.addWidget(self.progressbar)
        self.setLayout(ly)
        self.connect(self, self.change_maximum_signal, self.progressbar.setMaximum)
        self.connect(self, self.change_value_signal, self.progressbar.setValue)

    def run_import(self):
        collection = self.model.get_collection_getter()()
        self.emit(self.change_maximum_signal, len(collection))
        for i,row in enumerate(collection):
            new_entity_instance = self.admin.entity()
            for field_name, attributes in self.admin.get_columns():
                setattr(
                    new_entity_instance,
                    attributes['original_field'],
                    attributes['from_string'](getattr(row, field_name))
                )
            self.admin.add(new_entity_instance)
            self.admin.flush(new_entity_instance)
            self.emit(self.change_value_signal, i)

    def import_finished(self):
        self.progressbar.setMaximum(1)
        self.progressbar.setValue(1)
        self.emit(QtCore.SIGNAL('completeChanged()'))

    def isComplete(self):
        return self.progressbar.value() == self.progressbar.maximum()

    def initializePage(self):
        from camelot.view.model_thread import post
        self.progressbar.setMaximum(1)
        self.progressbar.setValue(0)
        self.emit(QtCore.SIGNAL('completeChanged()'))
        post(self.run_import, self.import_finished, self.import_finished)


class ImportWizard(QtGui.QWizard):
    """ImportWizard provides a two-step wizard for importing data as objects
    into Camelot.  To create a custom wizard, subclass this ImportWizard and
    overwrite its class attributes.

    To import a different file format, you probably need a custom
    collection_getter for this file type.
    """

    select_file_page = SelectFilePage
    data_preview_page = DataPreviewPage
    final_page = FinalPage
    collection_getter = CsvCollectionGetter
    window_title = _('Import CSV data')

    def __init__(self, parent=None, admin=None):
        """:param admin: camelot model admin"""
        from camelot.view.proxy.collection_proxy import CollectionProxy
        super(ImportWizard, self).__init__(parent)
        assert admin

        row_data_admin = RowDataAdminDecorator(admin)
        model = CollectionProxy(
            row_data_admin,
            lambda:[],
            row_data_admin.get_columns
        )

        self.addPage(SelectFilePage(parent=self))
        self.addPage(
            DataPreviewPage(
                parent=self,
                model=model,
                collection_getter=self.collection_getter
            )
        )
        self.addPage(FinalPage(parent=self, model=model))
        self.setWindowTitle(_(self.window_title))

        self.setOption(QtGui.QWizard.NoCancelButton)



# -*- coding: utf-8 -*-
from PyQt4 import QtGui

import logging
import unittest
import os

from camelot.test import ModelThreadTestCase, EntityViewsTest, SchemaTest
from PyQt4.QtGui import *
from PyQt4.QtCore import *

logger = logging.getLogger('view.unittests')

static_images_path = os.path.join(os.path.dirname(__file__), '..', 'doc', 'sphinx', 'source', '_static')

def create_getter(getable):
  
    def getter():
        return getable
    
    return getter
        
#class ProxyEntityTest(ModelThreadTestCase):
#  """Test the functionality of the proxies to perform CRUD operations on stand
#  alone data"""
#  def setUp(self):
#    ModelThreadTestCase.setUp(self)
#    from camelot.model.authentication import Person, PartyAddressRoleType
#    from camelot.view.proxy.queryproxy import QueryTableProxy
#    from camelot.view.application_admin import ApplicationAdmin
#    self.app_admin = ApplicationAdmin()
#    self.block = self.mt.post_and_block
#    person_admin = self.app_admin.getEntityAdmin(Person)
#    party_address_role_type_admin = self.app_admin.getEntityAdmin(PartyAddressRoleType)
#    self.person_proxy = QueryTableProxy(person_admin, lambda:Person.query, person_admin.get_fields)
#    self.party_address_role_type_proxy = QueryTableProxy(party_address_role_type_admin, lambda:PartyAddressRoleType.query, party_address_role_type_admin.get_fields)
#    # get the columns of the proxy
#    self.person_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.person_proxy.getColumns())))
#    self.rows_before_insert = 0
#    self.rows_after_insert = 0
#    self.rows_after_delete = 0
#    self.new_person = None
#    # make sure nothing is running in the model thread any more
#    self.block(lambda:None)
#  def listPersons(self):
#    from camelot.model.authentication import Person
#    
#    def create_list():
#      print '<persons>'
#      for p in Person.query.all():
#        print p.id, unicode(p)
#      print '</persons>'
#    
#    self.block(create_list)
#  def numberOfPersons(self):
#    return self.block(lambda:self.person_proxy.getRowCount())
#  def insertNewPerson(self, valid):
#    from camelot.model.authentication import Person
#    self.rows_before_insert = self.numberOfPersons()
#    self.new_person = self.block(lambda:Person(first_name={True:u'test', False:None}[valid], last_name='test'))
#    self.person_proxy.insertRow(0, create_getter(self.new_person))
#    self.rows_after_insert = self.numberOfPersons()
#    self.assertTrue( self.rows_after_insert > self.rows_before_insert)
#  def updateNewPerson(self):
#    rows_before_update = self.numberOfPersons()
#    self.assertNotEqual( self.block(lambda:self.new_person.last_name), u'Testers')
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
#    self.person_proxy.setData(index, lambda:u'Testers')
#    self.assertEqual( self.block(lambda:self.new_person.last_name), u'Testers')
#    self.assertEqual(rows_before_update, self.numberOfPersons())
#  def updateNewPersonToValid(self):
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
#    self.person_proxy.setData(index, lambda:u'test')
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
#    self.person_proxy.setData(index, lambda:u'test')      
#  def updateNewPersonToInvalid(self):
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
#    self.person_proxy.setData(index, lambda:None)
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
#    self.person_proxy.setData(index, lambda:None)      
#  def deleteNewPerson(self):
#    self.person_proxy.removeRow(self.rows_before_insert)
#    self.rows_after_delete = self.numberOfPersons()
#    self.assertTrue( self.rows_after_delete < self.rows_after_insert )
#  def testCreateDataWithoutRequiredFields(self):
#    """Create a record that has no required fields, so the creation
#    should be instantaneous as opposed to delayed until all required fields are filled"""
#    from camelot.model.authentication import PartyAddressRoleType
#    rows_before_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
#    new_party_address_role_type_proxy = self.block(lambda:PartyAddressRoleType())
#    self.party_address_role_type_proxy.insertRow(0, create_getter(new_party_address_role_type_proxy))
#    self.assertNotEqual( self.block(lambda:new_party_address_role_type_proxy.id), None )
#    rows_after_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
#    self.assertTrue( rows_after_insert > rows_before_insert)      
#  def testCreateUpdateDeleteValidData(self):
#    self.insertNewPerson(valid=True)
#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
#    self.updateNewPerson()
#    self.deleteNewPerson()
#  def testCreateUpdateDeleteInvalidData(self):
#    self.insertNewPerson(valid=False)
#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
#    self.updateNewPerson()
#    self.deleteNewPerson()
#  def testCreateInvalidDataUpdateToValidAndDelete(self):
#    self.insertNewPerson(valid=False)
#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
#    self.updateNewPersonToValid()
#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
#    self.deleteNewPerson()
#  def testCreateValidDataUpdateToInvalidAndDelete(self):
#    self.insertNewPerson(valid=False)
#    self.updateNewPersonToInvalid()
#    self.deleteNewPerson()

                
#class ProxyOneToManyTest(ProxyEntityTest):
#  """Test the functionality of the proxies to perform CRUD operations on related
#  data"""
#  def setUp(self):
#    super(ProxyOneToManyTest, self).setUp()
#    from elixir import session
#    from camelot.view.proxy.queryproxy import QueryTableProxy
#    from camelot.model.authentication import Country, City, Address, PartyAddress
#    party_address_admin = self.app_admin.getEntityAdmin(PartyAddress)
#    self.party_address_proxy = QueryTableProxy(party_address_admin, PartyAddress.query, party_address_admin.get_fields)
#    self.party_address_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.party_address_proxy.getColumns())))
#    self.country = self.block(lambda:Country(code=u'BE', name=u'Belgium'))
#    self.city = self.block(lambda:City(code=u'2000', name=u'Antwerp', country=self.country))
#    self.address = self.block(lambda:Address(street1=u'Teststreet', city=self.city))
#    self.block(lambda:session.flush([self.country, self.city, self.address]))
#  def insertRelatedPartyAddress(self, valid):
#    from camelot.model.authentication import PartyAddress
#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['addresses'])
#    self.block(lambda:None)
#    self.related_party_address_proxy = self.person_proxy.data(index, Qt.EditRole).toPyObject()
#    self.block(lambda:None)
#    self.related_party_address_proxy = self.person_proxy.data(index, Qt.EditRole).toPyObject()
#    self.assertNotEqual(self.related_party_address_proxy, None)
#    self.party_address = self.block({True:lambda:PartyAddress(address=self.address), False:lambda:PartyAddress()}[valid])
#    self.related_party_address_proxy.insertRow(0, create_getter(self.party_address))
#    self.block(lambda:None)
#  def testCreateBothValidDataUpdateDelete(self):
#    self.insertNewPerson(valid=True)
#    self.insertRelatedPartyAddress(valid=True)
#    self.assertNotEqual(self.party_address.id, None)
#    self.deleteNewPerson()
#  def testCreateValidPersonInvalidPartyAddressUpdateDelete(self):
#    self.insertNewPerson(valid=True)
#    self.insertRelatedPartyAddress(valid=False)
#    self.assertEqual(self.party_address.id, None)
#    self.deleteNewPerson()
#  def testCreateInvalidPersonValidPartyAddressUpdateDelete(self):
#    self.insertNewPerson(valid=False)
#    self.insertRelatedPartyAddress(valid=False)
#    self.assertEqual(self.party_address.id, None)
#    self.deleteNewPerson()
#  def tearDown(self):
#    from elixir import session
#    self.block(lambda:self.address.delete())
#    self.block(lambda:self.city.delete())
#    self.block(lambda:self.country.delete())
#    self.block(lambda:session.flush([self.country, self.city, self.address]))
      
class EditorsTest(ModelThreadTestCase):
  """
Test the basic functionality of the editors :

- get_value
- set_value
- support for ValueLoading
"""
  
  images_path = static_images_path
  
  from camelot.view.controls import editors
  from camelot.view.proxy import ValueLoading

  def test_DateEditor(self):
    import datetime
    editor = self.editors.DateEditor()
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( None )
    self.assertEqual( editor.get_value(), None )
    editor.set_value( datetime.date(1980, 12, 31) )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), datetime.date(1980, 12, 31) )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.DateEditor(editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( None )
    self.assertEqual( editor.get_value(), None )
    editor.set_value( datetime.date(1980, 12, 31) )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), datetime.date(1980, 12, 31) )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( datetime.date(2008, 3, 28) )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_TextLineEditor(self):
    editor = self.editors.TextLineEditor(parent=None, length=10)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( u'za coś tam' )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), u'za coś tam' )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.TextLineEditor(parent=None, length=10, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( u'za coś tam' )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), u'za coś tam' )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( u'editor.set_enabled() Test' )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
 
  def test_StarEditor(self):
    editor = self.editors.StarEditor(parent=None, maximum=5)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 4 )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), 4 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.StarEditor(parent=None, maximum=5, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 4 )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), 4 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 5 )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_SmileyEditor(self):
    editor = self.editors.SmileyEditor(parent=None)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 'face-devil-grin' )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), 'face-devil-grin' )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.SmileyEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 'face-kiss' )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), 'face-kiss' )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 'face-monkey' )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
      
  def test_BoolEditor(self):
    editor = self.editors.BoolEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( True )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), True )
    editor.set_value( False )
    self.assertEqual( editor.get_value(), False )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.BoolEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( True )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), True )
    editor.set_value( False )
    self.assertEqual( editor.get_value(), False )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( True )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_CodeEditor(self):
    editor = self.editors.CodeEditor(parent=None, parts=['AAA', '999'])
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( ['XYZ', '123'] )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), ['XYZ', '123'] )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading ) 
    editor = self.editors.CodeEditor(parent=None, parts=['AAA', '999'], editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( ['XYZ', '123'] )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), ['XYZ', '123'] )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( ['SAM', '321'] )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )    

  def test_ColorEditor(self):
    editor = self.editors.ColorEditor(parent=None, editable=True)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( (255, 200, 255, 255) )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), (255, 200, 255, 255) )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.ColorEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( (255, 200, 255, 255) )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), (255, 200, 255, 255) )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( (150, 200, 100, 255) )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
     
  def test_ColoredFloatEditor(self):
    editor = self.editors.ColoredFloatEditor(parent=None, editable=True)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 0.0 )
    self.assertEqual( editor.get_value(), 0.0 )    
    editor.set_value( 3.14 )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), 3.14 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.ColoredFloatEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 0.0 )
    self.assertEqual( editor.get_value(), 0.0 )    
    editor.set_value( 3.14 )
    self.grab_widget( editor,'disabled' )
    self.assertEqual( editor.get_value(), 3.14 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( -7.45 )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_ChoicesEditor(self):
    editor = self.editors.ChoicesEditor(parent=None, editable=True)
    choices1 = [(1,u'A'), (2,u'B'), (3,u'C')]
    editor.set_choices( choices1 )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 2 )
    self.assertEqual(editor.get_choices(), choices1 )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), 2 )
    # now change the choices
    choices2 = [(4,u'D'), (5,u'E'), (6,u'F')]
    editor.set_choices( choices2 )
    self.assertEqual(editor.get_choices(), choices2 + [(2,u'B')])
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.ChoicesEditor(parent=None, editable=False)
    editor.set_choices(choices1)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 2 )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), 2 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 1 )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_FileEditor(self):
    editor = self.editors.FileEditor(parent=None, editable=True)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( self.ValueLoading )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.FileEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( self.ValueLoading )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_DateTimeEditor(self):
    import datetime
    editor = self.editors.DateTimeEditor(parent=None, editable=True)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( datetime.datetime(2009, 7, 19, 21, 5, 10, 0) )
    self.assertEqual( editor.get_value(), datetime.datetime(2009, 7, 19, 21, 5, 0 ) )
    self.grab_widget( editor, 'editable' )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )    
    editor = self.editors.DateTimeEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( datetime.datetime(2009, 7, 19, 21, 5, 10, 0) )
    self.assertEqual( editor.get_value(), datetime.datetime(2009, 7, 19, 21, 5, 0 ) )
    self.grab_widget( editor, 'disabled' )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading ) 
    editor.set_value( datetime.datetime(2009, 5, 21, 16, 16, 5, 0) )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )   
       
  def test_FloatEditor(self):
    editor = self.editors.FloatEditor(parent=None, editable=True, prefix='prefix')
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 0.0 )
    self.assertEqual( editor.get_value(), 0.0 )    
    editor.set_value( 3.14 )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), 3.14 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.FloatEditor(parent=None, editable=False, suffix='suffix')
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 0.0 )
    self.assertEqual( editor.get_value(), 0.0 )
    editor.set_value( 3.14 )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), 3.14 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 5.45 )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_ImageEditor(self):
    editor = self.editors.ImageEditor(parent=None, editable=True)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    self.grab_widget( editor, 'editable' )
    editor = self.editors.ImageEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    self.grab_widget( editor, 'disabled' )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
 
  def test_IntegerEditor(self):
    editor = self.editors.IntegerEditor(parent=None, editable=True)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 0 )
    self.assertEqual( editor.get_value(), 0 )    
    editor.set_value( 3 )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), 3 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.IntegerEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 0 )
    self.assertEqual( editor.get_value(), 0 )    
    editor.set_value( 3 )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), 3 )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( 134 )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
 
  def test_RichTextEditor(self):
    editor = self.editors.RichTextEditor(parent=None)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( u'<h1>Rich Text Editor</h1>' )
    self.grab_widget( editor, 'editable' )
    self.assertTrue( u'Rich Text Editor' in editor.get_value() )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.RichTextEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( u'<h1>Rich Text Editor</h1>' )
    self.grab_widget( editor, 'disabled' )
    self.assertTrue( u'Rich Text Editor' in editor.get_value() )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( u'<h2>Rich Text Editor, set_enabled() test</h2>' )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_TimeEditor(self):
    import datetime
    editor = self.editors.TimeEditor(parent=None, editable=True)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( datetime.time(21, 5, 0) )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(), datetime.time(21, 5, 0) )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor = self.editors.TimeEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( datetime.time(21, 5, 0) )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(), datetime.time(21, 5, 0) )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( datetime.time(15, 6, 3) )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )
    
  def test_VirtualAddressEditor(self):
    editor = self.editors.VirtualAddressEditor(parent=None)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( ('im','test') )
    self.grab_widget( editor, 'editable' )
    self.assertEqual( editor.get_value(),  ('im','test') )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )  
    editor = self.editors.VirtualAddressEditor(parent=None, editable=False)
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( ('im','test') )
    self.grab_widget( editor, 'disabled' )
    self.assertEqual( editor.get_value(),  ('im','test') )
    editor.set_value( self.ValueLoading )
    self.assertEqual( editor.get_value(), self.ValueLoading )
    editor.set_value( ('email', 'test@conceptive.be') )
    editor.set_enabled( True )
    self.grab_widget( editor, 'set_enabled()_editable' )  
  
from camelot.view import forms
  
class FormTest(ModelThreadTestCase):
    
  images_path = static_images_path
  
  def setUp(self):
    ModelThreadTestCase.setUp(self)
    from camelot.view.controls.editors import TextLineEditor, DateEditor
    self.widgets = {
      'title':(QLabel('Title'), TextLineEditor(parent=None, editable=True)),
      'short_description':(QLabel('Short description'), TextLineEditor(parent=None, editable=True)),
      'director':(QLabel('Director'), TextLineEditor(parent=None, editable=True)),
      'release_date':(QLabel('Release date'), DateEditor(parent=None, editable=True)),
    }
    from elixir import entities
    self.entities = [e for e in entities]
    from camelot.admin.application_admin import ApplicationAdmin
    from camelot.model.authentication import Person
    self.app_admin = ApplicationAdmin()
    self.person_entity = Person
    self.collection_getter = lambda:[Person()]   
    
  def tearDown(self):
    #
    # The global list of entities should remain clean for subsequent tests
    #
    from elixir import entities
    for e in entities:
      if e not in self.entities:
        entities.remove(e)
    
  def test_form(self):
    from snippet.form.simple_form import Movie
    self.grab_widget(Movie.Admin.form_display.render(self.widgets))
    
  def test_tab_form(self):
    form = forms.TabForm([('First tab', ['title', 'short_description']),
                          ('Second tab', ['director', 'release_date'])])
    self.grab_widget(form.render(self.widgets))
    
  def test_group_box_form(self):
    form = forms.GroupBoxForm('Movie', ['title', 'short_description'])
    self.grab_widget(form.render(self.widgets))
    
  def test_grid_form(self):
    form = forms.GridForm([['title', 'short_description'], ['director','release_date']])
    self.grab_widget(form.render(self.widgets))
    
  def test_vbox_form(self):
    form = forms.VBoxForm([['title', 'short_description'], ['director', 'release_date']])
    self.grab_widget(form.render(self.widgets))
    
  def test_hbox_form(self):
    form = forms.HBoxForm([['title', 'short_description'], ['director', 'release_date']])
    self.grab_widget(form.render(self.widgets))
    
  def test_nested_form(self):
      from snippet.form.nested_form import Admin
      person_admin = Admin(self.app_admin, self.person_entity)
      self.grab_widget( person_admin.create_new_view() )
      
  def test_inherited_form(self):
      from snippet.form.inherited_form import InheritedAdmin
      person_admin = InheritedAdmin(self.app_admin, self.person_entity)
      self.grab_widget( person_admin.create_new_view() )
      
  def test_custom_layout(self):
      from snippet.form.custom_layout import Admin
      person_admin = Admin(self.app_admin, self.person_entity)
      self.grab_widget( person_admin.create_new_view() )      
                          
from camelot.admin import form_action
  
class FormActionTest(ModelThreadTestCase):
    
  images_path = static_images_path
  
  def test_print_html_form_action(self):
    action = form_action.PrintHtmlFormAction('Summary')
    widget = action.render(None, None)
    self.grab_widget(widget)
                            
class DelegateTest(unittest.TestCase):
  """Test the basic functionallity of the delegates :
- createEditor
- setEditorData
- setModelData
"""

  from camelot.view.controls import delegates
  from camelot.view.controls import editors
    
  def setUp(self):
    self.kwargs = dict(editable=True)
    self.option = QtGui.QStyleOptionViewItem()
    # set version to 5 to indicate the widget will appear on a
    # a form view and not on a table view, so it should not
    # set its background
    self.option.version = 5

  def grab_delegate(self, delegate, data, suffix='editable'):
    import sys
    from camelot.view.controls.tableview import TableWidget
    from PyQt4.QtCore import Qt
    
    model = QStandardItemModel(1, 1)
    index = model.index(0, 0, QModelIndex())
    model.setData(index, QVariant(data))
    model.setData(index, QVariant(QtGui.QColor('white')), Qt.BackgroundRole)
    
    option = QtGui.QStyleOptionViewItem()
    
    if suffix == 'editable':
      self.assertTrue(delegate.sizeHint(option, index).height()>1)
    
    tableview = TableWidget()
    tableview.setModel(model)
    tableview.setItemDelegate(delegate)
    
    test_case_name = sys._getframe(1).f_code.co_name[4:]
    
    for state_name, state in zip(('selected', 'unselected'), 
                                 (QStyle.State_Selected, QStyle.State_None)):
      tableview.adjustSize()
      
      if state == QStyle.State_Selected:
        tableview.selectionModel().select(index, QItemSelectionModel.Select)
      else:
        tableview.selectionModel().select(index, QItemSelectionModel.Clear)
      
      cell_size = tableview.visualRect(index).size()
      
      headers_size = QSize(tableview.verticalHeader().width(),
                           tableview.horizontalHeader().height())
      
      extra_size = QSize(tableview.verticalScrollBar().width(),
                         tableview.horizontalScrollBar().height())

      tableview.resize(cell_size + headers_size + extra_size)

      # TODO checks if path exists
      delegate_images_path = os.path.join(static_images_path, 'delegates')
      if not os.path.exists(delegate_images_path):
        os.makedirs(delegate_images_path)
      pixmap = QPixmap.grabWidget(tableview)
      pixmap.save(os.path.join(delegate_images_path, '%s_%s_%s.png'%(test_case_name, state_name, suffix)),
                  'PNG')
            
  def testPlainTextDelegate(self):
    delegate = self.delegates.PlainTextDelegate(parent=None,
                                                length=30,
                                                editable=True)
    editor = delegate.createEditor(None, self.option, None)
    self.assertEqual(editor.maxLength(), 30)
    self.assertTrue(isinstance(editor, self.editors.TextLineEditor))
    self.grab_delegate(delegate, 'Plain Text')
    delegate = self.delegates.PlainTextDelegate(parent=None,
                                                length=20,
                                                editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.TextLineEditor))
    self.grab_delegate(delegate, 'Plain Text', 'disabled')
    
  def testTextEditDelegate(self):
    from PyQt4.QtGui import QTextEdit
    delegate = self.delegates.TextEditDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, QTextEdit))
    self.grab_delegate(delegate, 'Plain Text')
    delegate = self.delegates.TextEditDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, QTextEdit))
    self.grab_delegate(delegate, 'Plain Text', 'disabled')

  def testRichTextDelegate(self):
    delegate = self.delegates.RichTextDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.RichTextEditor))
    self.grab_delegate(delegate, '<b>Rich Text</b>')
    delegate = self.delegates.RichTextDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.RichTextEditor))
    self.grab_delegate(delegate, '<b>Rich Text</b>', 'disabled')
    
  def testBoolDelegate(self):
    delegate = self.delegates.BoolDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.BoolEditor))
    self.grab_delegate(delegate, True)
    delegate = self.delegates.BoolDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.BoolEditor))
    self.grab_delegate(delegate, True, 'disabled')
  
  def testDateDelegate(self):
    from datetime import date
    delegate = self.delegates.DateDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.DateEditor))
    date = date.today()
    self.grab_delegate(delegate, date)
    delegate = self.delegates.DateDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.DateEditor))
    date = date.today()
    self.grab_delegate(delegate, date, 'disabled')
    #TODO  !!

  def testDateTimeDelegate(self):
    from datetime import datetime
    delegate = self.delegates.DateTimeDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.DateTimeEditor))
    DateTime = datetime.now()
    self.grab_delegate(delegate, DateTime)
    delegate = self.delegates.DateTimeDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.DateTimeEditor))
    self.grab_delegate(delegate, DateTime, 'disabled')

  def testTimeDelegate(self):
    from datetime import time
    delegate = self.delegates.TimeDelegate(parent=None, editable=True)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.TimeEditor))
    time = time(10, 30, 15)
    self.grab_delegate(delegate, time)
    delegate = self.delegates.TimeDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.TimeEditor))
    #time = time(10, 30, 15)
    self.grab_delegate(delegate, time, 'disabled')
    
  def testIntegerDelegate(self):
    delegate = self.delegates.IntegerDelegate(parent=None, editable=True)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.IntegerEditor))
    self.grab_delegate(delegate, 3)
    delegate = self.delegates.IntegerDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.IntegerEditor))
    self.grab_delegate(delegate, 0, 'disabled')

  def testIntervalsDelegate(self):
    from camelot.container import IntervalsContainer, Interval
    intervals = IntervalsContainer(0, 24, [Interval(8, 18, 'work', (255,0,0,255)), Interval(19, 21, 'play', (0,255,0,255))])
    delegate = self.delegates.IntervalsDelegate(parent=None, editable=True)
    self.grab_delegate(delegate, intervals)
    delegate = self.delegates.IntervalsDelegate(parent=None, editable=False)
    self.grab_delegate(delegate, intervals, 'disabled')
    
  def testFloatDelegate(self):
    from camelot.core.constants import camelot_minfloat, camelot_maxfloat
    delegate = self.delegates.FloatDelegate(parent=None, suffix='euro', editable=True)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.FloatEditor))
    self.assertEqual(editor.spinBox.minimum(), camelot_minfloat)
    self.assertEqual(editor.spinBox.maximum(), camelot_maxfloat)
    self.grab_delegate(delegate, 3.145)
    delegate = self.delegates.FloatDelegate(parent=None, prefix='prefix', editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.FloatEditor))
    self.assertEqual(editor.spinBox.minimum(), camelot_minfloat)
    self.assertEqual(editor.spinBox.maximum(), camelot_maxfloat)
    self.grab_delegate(delegate, 0, 'disabled')

  def testColoredFloatDelegate(self):
    delegate = self.delegates.ColoredFloatDelegate(parent=None, precision=3, editable=True)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.ColoredFloatEditor))
    self.grab_delegate(delegate, 3.14456)
    delegate = self.delegates.ColoredFloatDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.ColoredFloatEditor))
    self.grab_delegate(delegate, 3.1, 'disabled')
  
  def testStarDelegate(self):
    delegate = self.delegates.StarDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(delegate.maximum, 5)
    self.assertTrue(isinstance(editor, self.editors.StarEditor))
    self.grab_delegate(delegate, 5)
    delegate = self.delegates.StarDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(delegate.maximum, 5)
    self.assertTrue(isinstance(editor, self.editors.StarEditor))
    self.grab_delegate(delegate, 5, 'disabled')
    
  def testSmileyDelegate(self):
    delegate = self.delegates.SmileyDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.SmileyEditor))
    self.grab_delegate(delegate, 'face-glasses')
    delegate = self.delegates.SmileyDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.SmileyEditor))
    self.grab_delegate(delegate, 'face-glasses', 'disabled')

  def testFileDelegate(self):
    delegate = self.delegates.FileDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.FileEditor))

  def testColorDelegate(self):
    delegate = self.delegates.ColorDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.ColorEditor))
    color = [255, 255, 0]
    self.grab_delegate(delegate, color)
    delegate = self.delegates.ColorDelegate(parent=None, editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.ColorEditor))
    self.grab_delegate(delegate, color, 'disabled')
  
  def testCodeDelegate(self):
    delegate = self.delegates.CodeDelegate(parent=None, parts=['99','AA'], **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.CodeEditor))
    self.grab_delegate(delegate, ['76','AB'])
    delegate = self.delegates.CodeDelegate(parent=None, parts=['99','AA', '99', 'AA'], editable=False)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.CodeEditor))
    self.grab_delegate(delegate, ['76','AB', '12', '34'], 'disabled')
    
  def testCurrencyDelegate(self):
    delegate = self.delegates.CurrencyDelegate(parent=None, suffix='euro', **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.FloatEditor))
    self.grab_delegate(delegate, 1000000.12)
    delegate = self.delegates.CurrencyDelegate(parent=None, prefix='prefix', editable=False)
    self.grab_delegate(delegate, 1000000.12, 'disabled')    
  
  def testComboBoxDelegate(self):
    CHOICES = (('1','A'), ('2','B'), ('3','C'))
    delegate = self.delegates.ComboBoxDelegate(parent=None,
                                               choices=CHOICES,
                                               **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.ChoicesEditor))
    self.grab_delegate(delegate, 1)
    delegate = self.delegates.ComboBoxDelegate(parent=None,
                                               choices=CHOICES,
                                               editable=False)
    self.grab_delegate(delegate, 1, 'disabled')

  def testImageDelegate(self):
    delegate = self.delegates.ImageDelegate(parent=None, **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.ImageEditor))

  def testVirtualAddressDelegate(self):
    delegate = self.delegates.VirtualAddressDelegate(parent=None,
                                                     **self.kwargs)
    editor = delegate.createEditor(None, self.option, None)
    self.assertTrue(isinstance(editor, self.editors.VirtualAddressEditor))
    self.grab_delegate(delegate, ('email', 'project-camelot@conceptive.be'))
    delegate = self.delegates.VirtualAddressDelegate(parent=None, editable=False)
    self.grab_delegate(delegate, ('email', 'project-camelot@conceptive.be'), 'disabled')

class FilterTest(ModelThreadTestCase):
  """Test the filters in the table view"""
  
  images_path = static_images_path
  
  def test_group_box_filter(self):
    from camelot.view import filters
    filter = filters.GroupBoxFilter('state')
    self.grab_widget(filter.render(None, 'Organization', [('Nokia',None), ('Apple',None)]))

  def test_combo_box_filter(self):
    from camelot.view import filters
    filter = filters.ComboBoxFilter('state')
    self.grab_widget(filter.render(None, 'Organization', [('Nokia',None), ('Apple',None)]))
    
class ControlsTest(ModelThreadTestCase):
    """Test some basic controls"""
    
    images_path = static_images_path
    
    def setUp(self):
        super(ControlsTest, self).setUp()
        from camelot.admin.application_admin import ApplicationAdmin
        self.app_admin = ApplicationAdmin()
          
    def test_table_view(self):
        from camelot.view.controls.tableview import TableView
        from camelot.model.authentication import Person
        widget = TableView(self.app_admin.get_entity_admin(Person))
        self.grab_widget(widget)
    
    def test_navigation_pane(self):
        from camelot.view.controls import navpane
        widget = navpane.NavigationPane(self.app_admin)
        self.grab_widget(widget)
      
    def test_main_window(self):
        from camelot.view.mainwindow import MainWindow
        widget = MainWindow(self.app_admin)
        self.grab_widget(widget)
        
    def test_status_bar(self):
        from camelot.view.controls.statusbar import StatusBar
        status_bar = StatusBar(None)
        status_bar.busy_widget.set_busy(True)
        self.grab_widget(status_bar)
    
class CamelotEntityViewsTest(EntityViewsTest):
  """Test the views of all the Entity subclasses"""
  
  images_path = static_images_path
  
class SnippetsTest(ModelThreadTestCase):
    
    images_path = static_images_path
    
    def test_fields_with_actions(self):
        from snippet.fields_with_actions import Coordinate
        from camelot.view.proxy.collection_proxy import CollectionProxy
        coordinate = Coordinate()
        admin = Coordinate.Admin(None, Coordinate)
        proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
        form = admin.create_form_view('Coordinate', proxy, 0, None)
        self.grab_widget(form)
      
    def test_fields_with_tooltips(self):
        from snippet.fields_with_tooltips import Coordinate
        from camelot.view.proxy.collection_proxy import CollectionProxy
        coordinate = Coordinate()
        admin = Coordinate.Admin(None, Coordinate)
        proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
        form = admin.create_form_view('Coordinate', proxy, 0, None)
        self.grab_widget(form)
        
    def test_entity_validator(self):
        from camelot.view.proxy.collection_proxy import CollectionProxy
        from camelot.model.authentication import Person
        from camelot.admin.application_admin import ApplicationAdmin
        from snippet.entity_validator import PersonValidator, Admin
        app_admin = ApplicationAdmin()
        person_admin = Admin(app_admin, Person)
        proxy = CollectionProxy(person_admin, lambda:[Person()], person_admin.get_columns)
        validator = PersonValidator(person_admin, proxy)
        self.mt.post(lambda:validator.isValid(0))
        self.process()
        self.assertEqual(len(validator.validityMessages(0)), 3)
        self.grab_widget(validator.validityDialog(0, parent=None))
        
    def test_background_color(self):
        from camelot.view.proxy.collection_proxy import CollectionProxy
        from camelot.model.authentication import Person
        from camelot.admin.application_admin import ApplicationAdmin
        from snippet.background_color import Admin
        app_admin = ApplicationAdmin()
        person_admin = Admin(app_admin, Person)
        proxy = CollectionProxy(person_admin, lambda:[Person(first_name='John', last_name='Cleese'),
                                                      Person(first_name='eric', last_name='Idle')],
                                                       person_admin.get_columns)
        from camelot.view.controls.editors.one2manyeditor import One2ManyEditor
        editor = One2ManyEditor(admin=person_admin)
        editor.set_value(proxy)
        self.process()
        self.grab_widget(editor)
  
class CamelotSchemaTest(SchemaTest):
  
    images_path = static_images_path
  

if __name__ == '__main__':
    logger.info('running unit tests')
    app = get_application()
    runner=unittest.TextTestRunner(verbosity=2)
    snippets_test =  unittest.makeSuite(SnippetsTest, 'test')
    runner.run(snippets_test)
#  proxy_test =  unittest.makeSuite(ProxyEntityTest, 'test')
#  runner.run(proxy_test)
#  proxy_one_to_many_test =  unittest.makeSuite(ProxyOneToManyTest, 'test')
#  runner.run(proxy_one_to_many_test)
    schema_test =  unittest.makeSuite(CamelotSchemaTest, 'test')
    runner.run(schema_test)
    
    form_action_test =  unittest.makeSuite(FormActionTest, 'test')
    runner.run(form_action_test)
    editors_test =  unittest.makeSuite(EditorsTest, 'test')
    runner.run(editors_test)
    editor_test =  unittest.makeSuite(DelegateTest, 'test')
    runner=unittest.TextTestRunner(verbosity=2)
    runner.run(editor_test)
    controls_test = unittest.makeSuite(ControlsTest, 'test')
    runner.run(controls_test)
    form_test = unittest.makeSuite(FormTest, 'test')
    runner.run(form_test)
    filter_test = unittest.makeSuite(FilterTest, 'test')
    runner.run(filter_test)    
    entity_views_test = unittest.makeSuite(CamelotEntityViewsTest, 'test')
    runner.run(entity_views_test)




from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

from customeditor import CustomEditor

class ColorEditor(CustomEditor):
  
    def __init__(self, parent=None, editable=True, **kwargs):
        CustomEditor.__init__(self, parent)
        layout = QtGui.QVBoxLayout(self)
        layout.setSpacing(0)
        layout.setMargin(0)
        self.color_button = QtGui.QPushButton(parent)
        self.color_button.setMaximumSize(QtCore.QSize(20, 20))
        layout.addWidget(self.color_button)
        if editable:
            self.connect(self.color_button,
                         QtCore.SIGNAL('clicked(bool)'),
                         self.buttonClicked)
        self.setLayout(layout)
        self._color = None
    
    def get_value(self):
        color = self.getColor()
        if color:
            value = (color.red(), color.green(), color.blue(), color.alpha())
        else:
            value = None
        return CustomEditor.get_value(self) or value
          
    def set_value(self, value):
        value = CustomEditor.set_value(self, value)
        if value:
            color = QtGui.QColor()
            color.setRgb(*value)
            self.setColor(color)
        else:
            self.setColor(value)    
          
    def getColor(self):
        return self._color
      
    def set_enabled(self, editable=True):
        self.color_button.setEnabled(editable)
      
    def setColor(self, color):
        pixmap = QtGui.QPixmap(16, 16)
        if color:
            pixmap.fill(color)
        else:
            pixmap.fill(Qt.transparent)
        self.color_button.setIcon(QtGui.QIcon(pixmap))
        self._color = color
         
    def buttonClicked(self, raised):
        if self._color:
            color = QtGui.QColorDialog.getColor(self._color)
        else:
            color = QtGui.QColorDialog.getColor()
        if color.isValid() and color!=self._color:
            self.setColor(color)
            self.emit(QtCore.SIGNAL('editingFinished()'))



#  ============================================================================
#
#  Copyright (C) 2007-2008 Conceptive Engineering bvba. All rights reserved.
#  www.conceptive.be / project-camelot@conceptive.be
#
#  This file is part of the Camelot Library.
#
#  This file may be used under the terms of the GNU General Public
#  License version 2.0 as published by the Free Software Foundation
#  and appearing in the file LICENSE.GPL included in the packaging of
#  this file.  Please review the following information to ensure GNU
#  General Public Licensing requirements will be met:
#  http://www.trolltech.com/products/qt/opensource.html
#
#  If you are unsure which license is appropriate for your use, please
#  review the following information:
#  http://www.trolltech.com/products/qt/licensing.html or contact
#  project-camelot@conceptive.be.
#
#  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
#  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  For use of this library in commercial applications, please contact
#  project-camelot@conceptive.be
#
#  ============================================================================

"""
Tool to assist in the management of Camelot projects.

version_control

Puts the database under version control

eg : python manage.py console
launches a python console with the model being set up

"""

from code import InteractiveConsole
import sys

import settings

class FileCacher:
    "Cache the stdout text so we can analyze it before returning it"
    def __init__(self): self.reset()
    def reset(self): self.out = []
    def write(self,line): self.out.append(line)
    def flush(self):
        output = '\n'.join(self.out)
        self.reset()
        return output
    
class Shell(InteractiveConsole):
    "Wrapper around Python that can filter input/output to the shell"
    def __init__(self, locals={}):
        self.stdout = sys.stdout
        self.cache = FileCacher()
        InteractiveConsole.__init__(self, locals)
        return
    def get_output(self): sys.stdout = self.cache
    def return_output(self): sys.stdout = self.stdout
    def push(self,line):
        line = line.replace('\r','')
        self.get_output()
        # you can filter input here by doing something like
        # line = filter(line)
        InteractiveConsole.push(self,line)
        self.return_output()
        output = self.cache.flush()
        # you can filter the output here by doing something like
        # output = filter(output)
        print output # or do something else with it
        return 
    
def schema_display(image_path='schema.png'):
    from camelot.core.schema_display import create_uml_graph
    from sqlalchemy import orm
    from elixir import entities
    mappers = [orm.class_mapper(e) for e in entities]
    graph = create_uml_graph(mappers,
        show_operations=False, # not necessary in this case
        show_multiplicity_one=False # some people like to see the ones, some don't
    )
    graph.write_png(image_path)
      
def main():
    from optparse import OptionParser
    import camelot
    parser = OptionParser(__doc__, version=camelot.__version__)
    (_options, args) = parser.parse_args()
    if not args:
        parser.print_help()
    elif args[0]=='console':
        settings.setup_model()
        sh = Shell()
        sh.interact()
    elif args[0]=='schema_display':
        settings.setup_model()
        schema_display()
    elif args[0] in ('version_control', 'db_version', 'version', 'upgrade'):
        from migrate.versioning.repository import Repository
        from migrate.versioning.schema import ControlledSchema
        from sqlalchemy.exceptions import NoSuchTableError
        migrate_engine = settings.ENGINE()
        repository = Repository(settings.REPOSITORY)
        schema = None
        if args[0]=='version_control':
            migrate_connection = migrate_engine.connect()
            transaction = migrate_connection.begin()
            try:
                schema = ControlledSchema.create(migrate_engine, repository)
                transaction.commit()
            except:
                transaction.rollback()
                raise
            finally:
                migrate_connection.close()
            print 'database was put under version control'
        try:
            schema = ControlledSchema(migrate_engine, repository)
        except NoSuchTableError, _e:
            print 'database not yet under version control, use the version_control command first.'
        if schema:
            if args[0]=='db_version':
                print schema.version
            elif args[0]=='version':
                print repository.latest
            elif args[0]=='upgrade':
                migrate_connection = migrate_engine.connect()
                transaction = migrate_connection.begin()
                try:
                    schema.upgrade(args[1])
                    transaction.commit()
                    print schema.version
                except:
                    transaction.rollback()
                    raise
                finally:
                    migrate_connection.close()
    else:
        parser.print_help()          
             
if __name__ == '__main__':
    main()
